/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TERATERM.EXE, scroll buffer routines */

#include "teraterm.h"
#include "tttypes.h"
#include "ttwinman.h"

#include "buffer.h"

static HANDLE HCodeBuff = 0;
static HANDLE HAttrBuff = 0;
static HANDLE HAttrBuff2 = 0;
static BOOL BufferFull;

PCHAR CodeBuff;  /* Character code buffer */
PCHAR AttrBuff;  /* Attribute buffer */
PCHAR AttrBuff2; /* Color attr buffer */
PCHAR CodeLine;
PCHAR AttrLine;
PCHAR AttrLine2;
static LONG LinePtr;

LONG BufferSize;
int PageStart;
int BuffStartAbs, BuffEnd, BuffEndAbs;

int NumOfLines, NumOfColumns;
int NumOfLinesInBuff;
BOOL Select;
POINT SelectStart, SelectEnd, SelectEndOld;

BOOL BuffLock = FALSE;

void InitBuffer()
{
  int Ny;

  /* setup terminal */
  NumOfColumns = ts.TerminalWidth;
  NumOfLines = ts.TerminalHeight;

  /* setup window */
  if (ts.EnableScrollBuff>0)
  {
    if (ts.ScrollBuffMax < NumOfLines)
      ts.ScrollBuffMax = NumOfLines;
    Ny = ts.ScrollBuffMax;
  }
  else
    Ny = NumOfLines;

  if (! ChangeBuffer(NumOfColumns,Ny))
    PostQuitMessage(0);

  if (ts.EnableScrollBuff>0)
    ts.ScrollBuffMax = NumOfLinesInBuff;
}

BOOL ChangeBuffer(int Nx, int Ny)
{
  HANDLE HCodeNew, HAttrNew, HAttr2New;
  LONG NewSize;
  int NxCopy, NyCopy, i;
  PCHAR CodeDest, AttrDest, AttrDest2;
  PCHAR Ptr;
  LONG SrcPtr, DestPtr;
  BOOL LockOld;

  if ( Nx > BuffXMax ) Nx = BuffXMax;
  if ( Ny > BuffYMax ) Ny = BuffYMax;
 
  if ( (LONG)Nx * (LONG)Ny > BuffSizeMax )
    Ny = BuffSizeMax / Nx;

  NewSize = (LONG)Nx * (LONG)Ny;

  HCodeNew = GlobalAlloc(GMEM_MOVEABLE, NewSize);
  if ( HCodeNew==0 ) return FALSE;
  Ptr = GlobalLock(HCodeNew);
  if ( Ptr==NULL )
  {
    GlobalFree(HCodeNew);
    return FALSE;
  }
  CodeDest = Ptr;

  HAttrNew = GlobalAlloc(GMEM_MOVEABLE, NewSize);
  if ( HAttrNew==0 )
  {
    GlobalFree(HCodeNew);
    return FALSE;
  }
  Ptr = GlobalLock(HAttrNew);
  if ( Ptr==NULL )
  {
    GlobalFree(HCodeNew);
    GlobalFree(HAttrNew);
    return FALSE;
  }
  AttrDest = Ptr;

  HAttr2New = GlobalAlloc(GMEM_MOVEABLE, NewSize);
  if ( HAttr2New==0 )
  {
    GlobalFree(HCodeNew);
    GlobalFree(HAttrNew);
    return FALSE;
  }
  Ptr = GlobalLock(HAttr2New);
  if ( Ptr==NULL )
  {
    GlobalFree(HCodeNew);
    GlobalFree(HAttrNew);
    GlobalFree(HAttr2New);
    return FALSE;
  }
  AttrDest2 = Ptr;

  memset(&CodeDest[0], 0x20, NewSize);
  memset(&AttrDest[0], AttrDefault, NewSize);
  memset(&AttrDest2[0], AttrDefault2, NewSize);
  if ( HCodeBuff!=0 )
  {
    if ( NumOfColumns > Nx )
      NxCopy = Nx;
    else NxCopy = NumOfColumns;
    if ( BuffEnd > Ny )
      NyCopy = Ny;
    else NyCopy = BuffEnd;
    LockOld = BuffLock;
    LockBuffer();
    SrcPtr = GetLinePtr(BuffEnd-NyCopy);
    DestPtr = 0;
    for (i = 1 ; i <= NyCopy ; i++)
    {
      memcpy(&CodeDest[DestPtr],&CodeBuff[SrcPtr],NxCopy);
      memcpy(&AttrDest[DestPtr],&AttrBuff[SrcPtr],NxCopy);
      memcpy(&AttrDest2[DestPtr],&AttrBuff2[SrcPtr],NxCopy);
      SrcPtr = NextLinePtr(SrcPtr);
      DestPtr = DestPtr + (LONG)Nx;
    }
    FreeBuffer();
  }
  else {
    LockOld = FALSE;
    NyCopy = NumOfLines;
    Select = FALSE;
  }

  if ( Select )
  {
    SelectStart.y =
      SelectStart.y - BuffEnd + NyCopy;
    SelectEnd.y =
      SelectEnd.y - BuffEnd + NyCopy;
    if ( SelectStart.y < 0 )
    {
      SelectStart.y = 0;
      SelectStart.x = 0;
    }
    if ( SelectEnd.y<0 )
    {
      SelectEnd.x = 0;
      SelectEnd.y = 0;
    }

    Select = (SelectEnd.y > SelectStart.y) ||
                 ((SelectEnd.y=SelectStart.y) &&
                 (SelectEnd.x > SelectStart.x));
  }

  HCodeBuff = HCodeNew;
  HAttrBuff = HAttrNew;
  HAttrBuff2 = HAttr2New;
  BufferSize = NewSize;
  NumOfLinesInBuff = Ny;
  BuffStartAbs = 0;
  BuffEnd = NyCopy;

  if ( BuffEnd==NumOfLinesInBuff )
  {
    BufferFull = TRUE;
    BuffEndAbs = 0;
  }
  else {
    BufferFull = FALSE;
    BuffEndAbs = BuffEnd;
  }
  PageStart = BuffEnd - NumOfLines;

  LinePtr = 0;
  if ( LockOld )
  {
    CodeBuff = (PCHAR)GlobalLock(HCodeBuff);
    AttrBuff = (PCHAR)GlobalLock(HAttrBuff);
    AttrBuff2 = (PCHAR)GlobalLock(HAttrBuff2);
    CodeLine = CodeBuff;
    AttrLine = AttrBuff;
    AttrLine2 = AttrBuff2;
  }
  else {
    GlobalUnlock(HCodeNew);
    GlobalUnlock(HAttrNew);
  }
  BuffLock = LockOld;

  return TRUE;
}

void LockBuffer()
{
  if ( BuffLock ) return;
  CodeBuff = (PCHAR)GlobalLock(HCodeBuff);
  AttrBuff = (PCHAR)GlobalLock(HAttrBuff);
  AttrBuff2 = (PCHAR)GlobalLock(HAttrBuff2);
  CodeLine = &CodeBuff[LinePtr];
  AttrLine = &AttrBuff[LinePtr];
  AttrLine2 = &AttrBuff2[LinePtr];
  BuffLock = TRUE;
}

void UnlockBuffer()
{
  if ( ! BuffLock ) return;
  GlobalUnlock(HCodeBuff);
  GlobalUnlock(HAttrBuff);
  GlobalUnlock(HAttrBuff2);
  BuffLock = FALSE;
}

void FreeBuffer()
{
  UnlockBuffer();
  GlobalFree(HCodeBuff);
  GlobalFree(HAttrBuff);
  GlobalFree(HAttrBuff2);
}

void ResetBuff()
{
  PageStart = 0;
  BuffStartAbs = 0;
  BuffEnd = NumOfLines;
  if ( NumOfLines==NumOfLinesInBuff )
  {
    BufferFull = TRUE;
    BuffEndAbs = 0;
  }
  else {
    BufferFull = FALSE;
    BuffEndAbs = NumOfLines;
  }

  SelectStart.x = 0;
  SelectStart.y = 0;
  SelectEnd = SelectStart;
  SelectEndOld = SelectStart;
  Select = FALSE;

  NewLine(0);
  memset(&CodeBuff[0],0x20,BufferSize);
  memset(&AttrBuff[0],AttrDefault,BufferSize);
  memset(&AttrBuff2[0],AttrDefault2,BufferSize);
}

LONG GetLinePtr(int Line)
{
  LONG Ptr;

  Ptr = (LONG)(BuffStartAbs + Line) *
        (LONG)(NumOfColumns);
  if ( Ptr >= BufferSize )
    Ptr = Ptr - BufferSize;
  return Ptr;
}

LONG NextLinePtr(LONG Ptr)
{
  Ptr = Ptr + (LONG)NumOfColumns;
  if ( Ptr >= BufferSize )
    Ptr = Ptr - BufferSize;
  return Ptr;
}

LONG PrevLinePtr(LONG Ptr)
{
  Ptr = Ptr - (LONG)NumOfColumns;
  if ( Ptr < 0 ) Ptr = Ptr + BufferSize;
  return Ptr;
}

void NewLine(int Line)
{
  LinePtr = GetLinePtr(Line);
  CodeLine = &CodeBuff[LinePtr];
  AttrLine = &AttrBuff[LinePtr];
  AttrLine2 = &AttrBuff2[LinePtr];
}

void NextLine()
{
  LinePtr = NextLinePtr(LinePtr);
  CodeLine = &CodeBuff[LinePtr];
  AttrLine = &AttrBuff[LinePtr];
  AttrLine2 = &AttrBuff2[LinePtr];
}

void PrevLine()
{
  LinePtr = PrevLinePtr(LinePtr);
  CodeLine = &CodeBuff[LinePtr];
  AttrLine = &AttrBuff[LinePtr];
  AttrLine2 = &AttrBuff2[LinePtr];
}

void BuffScroll(int Count)
{
  int i;
  LONG TmpPtr;

  TmpPtr = GetLinePtr(BuffEnd);
  for (i = 1 ; i <= Count ; i++)
  {
    memset(&CodeBuff[TmpPtr],0x20,NumOfColumns);
    memset(&AttrBuff[TmpPtr],AttrDefault,NumOfColumns);
    memset(&AttrBuff2[TmpPtr],AttrDefault2,NumOfColumns);
    TmpPtr = NextLinePtr(TmpPtr);
    BuffEndAbs++;
    if ( BuffEndAbs >= NumOfLinesInBuff )
      BuffEndAbs = 0;
    if ( BufferFull )
    {
      BuffStartAbs = BuffEndAbs;
      if ( Select )
      {
        SelectStart.y--;
        SelectEnd.y--;
        if ( SelectStart.y<0 )
        {
          SelectStart.x = 0;
          SelectStart.y = 0;
        }
        if ( SelectEnd.y<0 )
        {
          SelectEnd.x = 0;
          SelectEnd.y = 0;
        }
        Select =
          (SelectEnd.y > SelectStart.y) ||
          ((SelectEnd.y==SelectStart.y) &&
           (SelectEnd.x > SelectStart.x));
      }
    }
    else {
      PageStart++;
      BuffEnd++;
      BufferFull = (BuffEnd == NumOfLinesInBuff);
    }
    LinePtr = NextLinePtr(LinePtr);
  }

  CodeLine = &CodeBuff[LinePtr];
  AttrLine = &AttrBuff[LinePtr];
  AttrLine2 = &AttrBuff2[LinePtr];
}
