/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TERATERM.EXE, Communication routines */
#include "teraterm.h"
#include "tttypes.h"
#include <process.h>
#include "ttlib.h"
#include "ttcommon.h"
#include "ttwsk.h"

#define ErrorCaption "Tera Term: Error"
#define ErrorCantConn "Cannot connect the host"
#define CommInQueSize 8192
#define CommOutQueSize 2048
#define CommXonLim 2048
#define CommXoffLim 2048
#define READENDNAME "ReadEnd"
#define WRITENAME "Write"
#define READNAME "Read"

static HANDLE ReadEnd;
static OVERLAPPED wol, rol;

/* reset a serial port which is already open */
void CommResetSerial(PTTSet ts, PComVar cv)
{
  DCB dcb;
  DWORD DErr;
  COMMTIMEOUTS ctmo;

  if (! cv->Open ||
      (cv->PortType != IdSerial)) return;

  ClearCommError(cv->ComID,&DErr,NULL);
  SetupComm(cv->ComID,CommInQueSize,CommOutQueSize);
  /* flush input que */
  PurgeComm(cv->ComID, PURGE_TXABORT | PURGE_RXABORT |
    PURGE_TXCLEAR | PURGE_RXCLEAR);
  cv->InBuffCount = 0;
  cv->InPtr = 0;
  cv->OutBuffCount = 0;
  cv->OutPtr = 0;

  cv->DelayPerChar = ts->DelayPerChar;
  cv->DelayPerLine = ts->DelayPerLine;

  memset(&ctmo,0,sizeof(ctmo));
  ctmo.ReadIntervalTimeout = MAXDWORD;
  ctmo.WriteTotalTimeoutConstant = 500;
  SetCommTimeouts(cv->ComID,&ctmo);

  memset(&dcb,0,sizeof(DCB));
  dcb.DCBlength = sizeof(DCB);
  switch (ts->Baud) {
    case IdBaud110: dcb.BaudRate = 110; break;
    case IdBaud300: dcb.BaudRate = 300; break;
    case IdBaud600: dcb.BaudRate = 600; break;
    case IdBaud1200: dcb.BaudRate = 1200; break;
    case IdBaud2400: dcb.BaudRate = 2400; break;
    case IdBaud4800: dcb.BaudRate = 4800; break;
    case IdBaud9600: dcb.BaudRate = 9600; break;
    case IdBaud19200: dcb.BaudRate = 19200; break;
    case IdBaud38400: dcb.BaudRate = 38400; break;
    case IdBaud57600: dcb.BaudRate = 57600; break;
    case IdBaud115200: dcb.BaudRate = 115200; break;
  }
  dcb.fBinary = TRUE;
  switch (ts->Parity) {
    case IdParityEven:
      dcb.fParity = TRUE;
      dcb.Parity = EVENPARITY;
      break;
    case IdParityOdd:
      dcb.fParity = TRUE;
      dcb.Parity = ODDPARITY;
      break;
    case IdParityNone:
      dcb.Parity = NOPARITY;
      break;
  }

  dcb.fDtrControl = DTR_CONTROL_ENABLE;
  dcb.fRtsControl = RTS_CONTROL_ENABLE;
  switch (ts->Flow) {
    case IdFlowX:
      dcb.fOutX = TRUE;
      dcb.fInX = TRUE;
      dcb.XonLim = CommXonLim;
      dcb.XoffLim = CommXoffLim;
      dcb.XonChar = XON;
      dcb.XoffChar = XOFF;
      break;
    case IdFlowHard:
      dcb.fOutxCtsFlow = TRUE;
      dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
      break;
  }

  switch (ts->DataBit) {
    case IdDataBit7: dcb.ByteSize = 7; break;
    case IdDataBit8: dcb.ByteSize = 8; break;
  }
  switch (ts->StopBit) {
    case IdStopBit1: dcb.StopBits = ONESTOPBIT; break;
    case IdStopBit2: dcb.StopBits = TWOSTOPBITS; break;
  }
  SetCommState(cv->ComID, &dcb);

  /* enable receive request */
  SetCommMask(cv->ComID,0);
  SetCommMask(cv->ComID,EV_RXCHAR);
}

void CommOpen(HWND HW, PTTSet ts, PComVar cv)
{
  BYTE COMFlag;
  int Err;
  char ErrMsg[21];
  char P[50];

  HANDLE HAsync;
  MSG Msg;
  char HEntBuff[MAXGETHOSTSTRUCT];
  u_long addr;
  SOCKADDR_IN saddr;

  BOOL InvalidHost;
  BOOL BBuf;

  /* initialize ComVar */
  cv->InBuffCount = 0;
  cv->InPtr = 0;
  cv->OutBuffCount = 0;
  cv->OutPtr = 0;
  cv->HWin = HW;
  cv->Ready = FALSE;
  cv->Open = FALSE;
  cv->PortType = ts->PortType;
  cv->ComPort = 0;
  cv->RetryCount = 0;
  cv->s = INVALID_SOCKET;
  cv->ComID = INVALID_HANDLE_VALUE;
  cv->CanSend = TRUE;
  cv->RRQ = FALSE;
  cv->SendKanjiFlag = FALSE;
  cv->SendCode = IdASCII;
  cv->EchoKanjiFlag = FALSE;
  cv->EchoCode = IdASCII;
  cv->Language = ts->Language;
  cv->CRSend = ts->CRSend;
  cv->KanjiCodeEcho = ts->KanjiCode;
  cv->JIS7KatakanaEcho = ts->JIS7Katakana;
  cv->KanjiCodeSend = ts->KanjiCodeSend;
  cv->JIS7KatakanaSend = ts->JIS7KatakanaSend;
  cv->KanjiIn = ts->KanjiIn;
  cv->KanjiOut = ts->KanjiOut;
  cv->DelayFlag = TRUE;
  cv->DelayPerChar = ts->DelayPerChar;
  cv->DelayPerLine = ts->DelayPerLine;
  cv->TelBinRecv = FALSE;
  cv->TelBinSend = FALSE;
  cv->TelFlag = FALSE;
  cv->TelMode = FALSE;
  cv->IACFlag = FALSE;
  cv->TelCRFlag = FALSE;
  cv->TelCRSend = FALSE;
  cv->TelCRSendEcho = FALSE;
  cv->HLogBuf = 0;
  cv->HBinBuf = 0;
  cv->LogBuf = NULL;
  cv->BinBuf = NULL;
  cv->LogPtr = 0;
  cv->LStart = 0;
  cv->LCount = 0;
  cv->BinPtr = 0;
  cv->BStart = 0;
  cv->BCount = 0;
  cv->DStart = 0;
  cv->DCount = 0;
  cv->BinSkip = 0;

  if ( (ts->PortType!=IdSerial) && (strlen(ts->HostName)==0) )
  {
    PostMessage(cv->HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
    return;
  }

  switch (ts->PortType) {
    case IdTCPIP:
      if (! LoadWinsock() )
      {
        MessageBox(cv->HWin,"Cannot use winsock",ErrorCaption,
                   MB_TASKMODAL | MB_ICONEXCLAMATION);
        InvalidHost = TRUE;
      }
      else {
        cv->Open = TRUE;
        if ((ts->HostName[0] >= 0x30) && (ts->HostName[0] <= 0x39))
        {
          addr = Pinet_addr(ts->HostName);
          InvalidHost = (addr == 0xffffffff);
        }
        else {
	  HAsync = PWSAAsyncGetHostByName(HW,WM_USER+MsgGetHost,
            ts->HostName,HEntBuff,sizeof(HEntBuff));
	  if (HAsync == 0)
            InvalidHost = TRUE;
          else {
            do {
	      if (GetMessage(&Msg,0,0,0))
	      {
                if ((Msg.message == WM_SYSCOMMAND) &&
                    (Msg.wParam == SC_CLOSE) ||
		    (Msg.message == WM_COMMAND) &&
		    (LOWORD(Msg.wParam) == CM_FILEEXIT) ||
		    (Msg.message == WM_CLOSE))
                { /* Exit when the user closes Tera Term */
		  PWSACancelAsyncRequest(HAsync);
                  PostMessage(HW,Msg.message,Msg.wParam,Msg.lParam);
                  return;
                }
                if (Msg.message != WM_USER+MsgGetHost)
                { /* Prosess messages */
                  TranslateMessage(&Msg);
                  DispatchMessage(&Msg);
                }
	      }
              else {
                PostMessage(HW,WM_QUIT,0,0);
                return;
              }
            } while (Msg.message!=WM_USER+MsgGetHost);

            InvalidHost = WSAGETASYNCERROR(Msg.lParam) != 0;
            if (! InvalidHost)
            {
              if (((PHOSTENT)HEntBuff)->h_addr_list != NULL)
		memcpy(&addr,((PHOSTENT)HEntBuff)->h_addr_list[0],sizeof(addr));
              else
                InvalidHost = TRUE;
            }
          }

        }

        if ( InvalidHost )
          MessageBox(cv->HWin,"Invalid host",ErrorCaption,
                     MB_TASKMODAL | MB_ICONEXCLAMATION);
        else {
          cv->s= Psocket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
          if (cv->s==INVALID_SOCKET)
            MessageBox(cv->HWin,ErrorCantConn,ErrorCaption,
                       MB_TASKMODAL | MB_ICONEXCLAMATION);
          else {
            BBuf = TRUE;
            Psetsockopt(cv->s,SOL_SOCKET,SO_OOBINLINE,(char *)&BBuf,sizeof(BBuf));

            PWSAAsyncSelect(cv->s,cv->HWin,WM_USER+MsgComOpen, FD_CONNECT);
            saddr.sin_family = AF_INET;
            saddr.sin_port = Phtons(ts->TCPPort);
            saddr.sin_addr.s_addr = addr;
            memset(saddr.sin_zero,0,8);

            Err = Pconnect(cv->s,(LPSOCKADDR)&saddr,sizeof(saddr));
            if (Err!=0 ) Err = PWSAGetLastError();
            if (Err==WSAEWOULDBLOCK )
            {
              /* Do nothing */
            }
            else if (Err!=0 )
              PostMessage(cv->HWin, WM_USER+MsgComOpen,0,
                          MAKELONG(FD_CONNECT,Err));
          }
        }
      }
      cv->TelFlag = (ts->Telnet > 0);
      break;

    case IdSerial:
      strcpy(P,"COM");
      P[3] = (char)(0x30+ts->ComPort);
      P[4] = 0;
      cv->ComID =
	CreateFile(P,GENERIC_READ | GENERIC_WRITE,
          0,NULL,OPEN_EXISTING,
	  FILE_FLAG_OVERLAPPED,NULL);

      if (cv->ComID == INVALID_HANDLE_VALUE )
      {
        strcpy(ErrMsg,"Cannot open ");
        strcat(ErrMsg,P);
        MessageBox(cv->HWin,ErrMsg,ErrorCaption,
                   MB_TASKMODAL | MB_ICONEXCLAMATION);
        InvalidHost = TRUE;
      }
      else {
        cv->Open = TRUE;
	cv->ComPort = ts->ComPort;
	CommResetSerial(ts,cv);

        /* notify to VT window that Comm Port is open */
        PostMessage(cv->HWin, WM_USER+MsgComOpen, 0, 0);

        InvalidHost = FALSE;

        COMFlag = GetCOMFlag();
        switch (ts->ComPort) {
          case 1: COMFlag = COMFlag | 1; break;
          case 2: COMFlag = COMFlag | 2; break;
          case 3: COMFlag = COMFlag | 4; break;
          case 4: COMFlag = COMFlag | 8; break;
        }
	SetCOMFlag(COMFlag);
      }
    break; /* end of "case IdSerial:" */

    case IdFile:
      cv->ComID = CreateFile(ts->HostName,GENERIC_READ,0,NULL,
	OPEN_EXISTING,0,NULL);
      InvalidHost = (cv->ComID == INVALID_HANDLE_VALUE);
      if ( InvalidHost )
        MessageBox(cv->HWin,"Cannot open file",ErrorCaption,
                   MB_TASKMODAL | MB_ICONEXCLAMATION);
      else {
        cv->Open = TRUE;
        PostMessage(cv->HWin, WM_USER+MsgComOpen, 0, 0);
      }
      break;
  } /* end of "switch" */

  if ( InvalidHost )
  {
    PostMessage(cv->HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
    if ( (ts->PortType==IdTCPIP) && cv->Open )
    {
      if ( cv->s!=INVALID_SOCKET ) Pclosesocket(cv->s);
      FreeWinsock();
    }
    return;
  }
}

void CommThread(void *arg)
{
  DWORD Evt;
  PComVar cv = (PComVar)arg;
  DWORD DErr;
  HANDLE REnd;

  REnd = OpenEvent(EVENT_ALL_ACCESS,FALSE, READENDNAME);
  while (TRUE) {
    if (WaitCommEvent(cv->ComID,&Evt,NULL))
    {
      if (! cv->Ready) _endthread();
      if (! cv->RRQ)
        PostMessage(cv->HWin, WM_USER+MsgComNotify, 0, FD_READ);
      WaitForSingleObject(REnd,INFINITE);
    }
    else {
      if (! cv->Ready) _endthread();
      ClearCommError(cv->ComID,&DErr,NULL);
    }
  }
}

void CommStart(PComVar cv, LONG lParam)
{
  char ErrMsg[31];

  if (! cv->Open ) return;
  if ( cv->Ready ) return;
  switch (cv->PortType) {
    case IdTCPIP:
      ErrMsg[0] = 0;
      switch (HIWORD(lParam)) {
        case WSAECONNREFUSED:
	  strcpy(ErrMsg,"Connection refused"); break;
        case WSAENETUNREACH:
	  strcpy(ErrMsg,"Network cannot be reached"); break;
        case WSAETIMEDOUT:
	  strcpy(ErrMsg,"Connection timed out"); break;
        default:
          strcpy(ErrMsg,ErrorCantConn);
      }
      if ( HIWORD(lParam)>0 )
      {
        MessageBox(cv->HWin,ErrMsg,ErrorCaption,
                   MB_TASKMODAL | MB_ICONEXCLAMATION);
        PostMessage(cv->HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
        return;
      }
      PWSAAsyncSelect(cv->s,cv->HWin,WM_USER+MsgComNotify, FD_READ | FD_OOB | FD_CLOSE);
      break;

    case IdSerial:
      ReadEnd = CreateEvent(NULL,FALSE,FALSE,READENDNAME);
      wol.hEvent = CreateEvent(NULL,TRUE,TRUE,WRITENAME);
      rol.hEvent = CreateEvent(NULL,TRUE,FALSE,READNAME);

      /* cleate the receiver thread */
      if (_beginthread(CommThread,NULL,8192,cv) == -1)
        MessageBox(cv->HWin,"Can't create thread",ErrorCaption,
                   MB_TASKMODAL | MB_ICONEXCLAMATION);
      break;
    case IdFile: cv->RRQ = TRUE; break;
  }
  cv->Ready = TRUE;
}

void CommClose(PComVar cv)
{
  BYTE COMFlag;

  if ( ! cv->Open ) return;
  /* close port & release resources */
  /* disable event message posting & flush buffer */
  
  if ( cv->HLogBuf!=0 )
  {
    if ( cv->LogBuf!=NULL )
      GlobalUnlock(cv->HLogBuf);
    GlobalFree(cv->HLogBuf);
  }
  if ( cv->HBinBuf!=0 )
  {
    if ( cv->BinBuf!=NULL )
      GlobalUnlock(cv->HBinBuf);
    GlobalFree(cv->HBinBuf);
  }
  cv->HLogBuf = 0;
  cv->LogBuf = NULL; 
  cv->LogPtr = 0;
  cv->HBinBuf = 0;
  cv->BinBuf = NULL;
  cv->BinPtr = 0;
  cv->LStart = 0;
  cv->LCount = 0;
  cv->BStart = 0;
  cv->BCount = 0;
  cv->DStart = 0;
  cv->DCount = 0;

  cv->RRQ = FALSE;
  cv->Ready = FALSE;
  cv->InPtr = 0;
  cv->InBuffCount = 0;
  cv->OutPtr = 0;
  cv->OutBuffCount = 0;
  cv->Open = FALSE;

  switch (cv->PortType) {
    case IdTCPIP:
      if ( cv->s!=INVALID_SOCKET )
        Pclosesocket(cv->s);
      cv->s = INVALID_SOCKET;
      FreeWinsock();
      break;
    case IdSerial:
      if ( cv->ComID != INVALID_HANDLE_VALUE )
      {
        CloseHandle(ReadEnd);
        CloseHandle(wol.hEvent);
        CloseHandle(rol.hEvent);
	PurgeComm(cv->ComID,
	  PURGE_TXABORT | PURGE_RXABORT |
	  PURGE_TXCLEAR | PURGE_RXCLEAR);
        EscapeCommFunction(cv->ComID,CLRDTR);
	SetCommMask(cv->ComID,0);
        CloseHandle(cv->ComID);
        COMFlag = GetCOMFlag();
        switch (cv->ComPort) {
          case 1: COMFlag = COMFlag & 0x0E; break;
          case 2: COMFlag = COMFlag & 0x0D; break;
          case 3: COMFlag = COMFlag & 0x0B; break;
          case 4: COMFlag = COMFlag & 0x07; break;
          default:
            COMFlag = 0;
        }
        SetCOMFlag(COMFlag); 
      }
      break;
    case IdFile:
      if ( cv->ComID != INVALID_HANDLE_VALUE )
	CloseHandle(cv->ComID);
      break;
  }
  cv->ComID = INVALID_HANDLE_VALUE;
  cv->PortType = 0;
}

void CommProcRRQ(PComVar cv)
{
  if ( ! cv->Ready ) return;
  /* disable receive request */
  switch (cv->PortType) {
    case IdTCPIP:
      PWSAAsyncSelect(cv->s,cv->HWin,WM_USER+MsgComNotify, FD_OOB | FD_CLOSE);
      break;
    case IdSerial: break;
  }
  cv->RRQ = TRUE;
  CommReceive(cv);
}

void CommReceive(PComVar cv)
{
  DWORD C, DErr;

  if (! cv->Ready || ! cv->RRQ ||
      (cv->InBuffCount>=InBuffSize)) return;

  /* Compact buffer */
  if ((cv->InBuffCount>0) && (cv->InPtr>0))
  {
    memmove(cv->InBuff,&(cv->InBuff[cv->InPtr]),cv->InBuffCount);
    cv->InPtr = 0;
  }

  if (cv->InBuffCount<InBuffSize)
  {
    switch (cv->PortType) {
      case IdTCPIP:
        C = Precv(cv->s, &(cv->InBuff[cv->InBuffCount]),
		 InBuffSize-cv->InBuffCount, 0);
        if (C == SOCKET_ERROR)
        {
          C = 0;
          PWSAGetLastError();
        }
        cv->InBuffCount = cv->InBuffCount + C;
        break;
      case IdSerial:
        do {
          ClearCommError(cv->ComID,&DErr,NULL);
	  if (! ReadFile(cv->ComID,&(cv->InBuff[cv->InBuffCount]),
                InBuffSize-cv->InBuffCount,&C,&rol))
  	  {
	    if (GetLastError() == ERROR_IO_PENDING)
	    {
              if (WaitForSingleObject(rol.hEvent, 1000) !=
		  WAIT_OBJECT_0)
                C = 0;
              else
                GetOverlappedResult(cv->ComID,&rol,&C,FALSE);
            }
	    else
	      C = 0;
          }
          cv->InBuffCount = cv->InBuffCount + C;
        } while ((C!=0) && (cv->InBuffCount<InBuffSize));
        ClearCommError(cv->ComID,&DErr,NULL);
	break;
      case IdFile:
	ReadFile(cv->ComID,&(cv->InBuff[cv->InBuffCount]),
		 InBuffSize-cv->InBuffCount,&C,NULL);
        cv->InBuffCount = cv->InBuffCount + C;
	break;
    }
  }

  if (cv->InBuffCount==0)
  {
    switch (cv->PortType) {
      case IdTCPIP:
        PWSAAsyncSelect(cv->s,cv->HWin,WM_USER+MsgComNotify, FD_READ | FD_OOB | FD_CLOSE);
	break;
      case IdSerial:
	cv->RRQ = FALSE;
        SetEvent(ReadEnd);
	return;
      case IdFile:
	PostMessage(cv->HWin, WM_USER+MsgComNotify, 0, FD_CLOSE);
	break;
    }
    cv->RRQ = FALSE;
    
  }
}

void CommSend(PComVar cv)
{
  int C, Max, delay;
  COMSTAT Stat;
  BYTE LineEnd;
  DWORD D, DErr;

  if ( ! cv->Ready ) return;

  if ((cv->OutBuffCount == 0) || (! cv->CanSend) ) return;

  /* Max num of bytes to be written */
  switch (cv->PortType) {
    case IdTCPIP:
      Max = cv->OutBuffCount;
      break;
    case IdSerial:
      ClearCommError(cv->ComID,&DErr,&Stat);
      Max = OutBuffSize - Stat.cbOutQue;
      break;
    case IdFile: Max = cv->OutBuffCount;
      break;
  }

  if ( Max==0 ) return;
  if ( Max > cv->OutBuffCount ) Max = cv->OutBuffCount;

  C = Max;
  delay = 0;

  if ( cv->DelayFlag && (cv->PortType==IdSerial) )
  {
    if ( cv->DelayPerLine > 0 )
    {
      if ( cv->CRSend==IdCR ) LineEnd = 0x0d;
                         else LineEnd = 0x0a;
      C = 1;
      if ( cv->DelayPerChar==0 )
        while ((C<Max) && (cv->OutBuff[cv->OutPtr+C-1]!=LineEnd))
          C++;
      if ( cv->OutBuff[cv->OutPtr+C-1]==LineEnd )
	delay = cv->DelayPerLine;
      else delay = cv->DelayPerChar;
    }
    else if ( cv->DelayPerChar > 0 )
    {
      C = 1;
      delay = cv->DelayPerChar;
    }
  }

  /* Write to comm driver/Winsock */
  switch (cv->PortType) {
    case IdTCPIP:
      D = Psend(cv->s, &(cv->OutBuff[cv->OutPtr]), C, 0);
      if ( D==SOCKET_ERROR ) /* if error occurs */
      {
        PWSAGetLastError(); /* Clear error */
        D = 0;
      }
      break;

    case IdSerial:
      if (! WriteFile(cv->ComID,&(cv->OutBuff[cv->OutPtr]),C,&D,&wol))
      {
	if (GetLastError() == ERROR_IO_PENDING)
	{
	  if (WaitForSingleObject(wol.hEvent,1000) !=
	      WAIT_OBJECT_0)
	    D = C;	/* Time out, ignore data */
	  else
	    GetOverlappedResult(cv->ComID,&wol,&D,FALSE);
	}
	else /* I/O error */
	  D = C; /* ignore error */
      }
      ClearCommError(cv->ComID,&DErr,&Stat);
      break;
    case IdFile: D = C; break;
  }

  cv->OutBuffCount = cv->OutBuffCount - D;
  if ( cv->OutBuffCount==0 )
    cv->OutPtr = 0;
  else
    cv->OutPtr = cv->OutPtr + D;

  if ( (C==D) && (delay>0) )
  {
    cv->CanSend = FALSE;
    SetTimer(cv->HWin, IdDelayTimer, delay, NULL);
  }
}

void CommSendBreak(PComVar cv)
/* for only serial ports */
{
  MSG DummyMsg;

  if ( ! cv->Ready ) return;

  switch (cv->PortType) {
    case IdSerial:
      /* Set com port into a break state */
      SetCommBreak(cv->ComID);

      /* pause for 1sec */
      if ( SetTimer(cv->HWin, IdBreakTimer, 1000, NULL) != 0 )
      GetMessage(&DummyMsg,cv->HWin,WM_TIMER,WM_TIMER);

      /* Set com port into a nonbreak state */
      ClearCommBreak(cv->ComID);
      break;
  }
}

