/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TERATERM.EXE, Printing routines */
#include "teraterm.h"
#include "tttypes.h"
#include <commdlg.h>
#include "ttftypes.h"
#include "filesys.h"
#include "ttwinman.h"
#include "buffer.h"
#include "vtdisp.h"

#define IdPrnAbortDlg 100

class CPrnAbortDlg : public CDialog
{
public:
  BOOL Create();

// Dialog Data
  //{{AFX_DATA(CPrnAbortDlg)
    enum { IDD = IdPrnAbortDlg };
	// NOTE: the ClassWizard will add data members here
  //}}AFX_DATA

protected:
  virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
};

  PRINTDLG PrnDlg;
extern "C" {
  HDC PrintDC;
}
  BOOL Printing = FALSE;
  BOOL PrintAbort = FALSE;

/* printer emulation */
#define PrnFName "PRINT.DAT"
#define IdPrnNormal 1
#define IdPrnESC 2
#define IdPrnCSI 3
#define IdPrnCSIi 4

extern "C" {
BOOL AutoPrintMode = FALSE;
BOOL PrinterMode = FALSE;
int PrnParseMode;
BOOL PrintEX;
int HPrnFile = 0;
}
static BOOL DB_CSI;
static BYTE PCM;

  CPrnAbortDlg *PrnAbortDlg;
  HWND HPrnAbortDlg;

/* Print Abortion Call Back Function */
BOOL CALLBACK PrnAbortProc(HDC PDC, int Code)
{
  MSG m;

  while ((! PrintAbort) && PeekMessage(&m, 0,0,0, PM_REMOVE))
    if ((HPrnAbortDlg==NULL) || (! IsDialogMessage(HPrnAbortDlg, &m)))
    {
      TranslateMessage(&m);
      DispatchMessage(&m);
    }

  if (PrintAbort)
  {
    if (PrnAbortDlg != NULL) PrnAbortDlg->DestroyWindow();
    HPrnAbortDlg = NULL;
    PrnAbortDlg = NULL;
  }

  return (! PrintAbort);
}

extern "C" {
BOOL PrnBox(HWND HWin, PBOOL Sel)
{
  /* initialize TPrintDlg record */
  memset(&PrnDlg, 0, sizeof(PRINTDLG));
  PrnDlg.lStructSize = sizeof(PRINTDLG);
  PrnDlg.hwndOwner = HWin;
  PrnDlg.Flags = PD_RETURNDC | PD_NOPAGENUMS | PD_SHOWHELP;
  if (! *Sel)
    PrnDlg.Flags = PrnDlg.Flags | PD_NOSELECTION;
  PrnDlg.nCopies = 1;

  /* 'Print' dialog box */
  if (! PrintDlg(&PrnDlg)) return FALSE; /* if 'Cancel' button clicked, exit */
  if (PrnDlg.hDC == NULL) return FALSE;
  PrintDC = PrnDlg.hDC;
  *Sel = (PrnDlg.Flags & PD_SELECTION) != 0;
  return TRUE;
}
}

extern "C" {
BOOL PrnStart(LPSTR DocumentName)
{
  DOCINFO Doc;
  char DocName[32];

  Printing = FALSE;
  PrintAbort = FALSE;

  PrnAbortDlg = new CPrnAbortDlg();
  if (PrnAbortDlg==NULL) return FALSE;
  PrnAbortDlg->Create();

  SetAbortProc(PrintDC,PrnAbortProc);

  Doc.cbSize = sizeof(DOCINFO);
  strncpy(DocName,DocumentName,31);
  Doc.lpszDocName = DocName;
  Doc.lpszOutput = NULL;
  if (StartDoc(PrintDC, &Doc) > 0)
    Printing = TRUE;
  else
    if (PrnAbortDlg != NULL) PrnAbortDlg->DestroyWindow();

  return Printing;
}
}

extern "C" {
void PrnStop()
{
  if (Printing)
  {
    EndDoc(PrintDC);
    DeleteDC(PrintDC);
    Printing = FALSE;
  }
  if (PrnAbortDlg != NULL)
  {
    PrnAbortDlg->DestroyWindow();
    PrnAbortDlg = NULL;
  }
}
}

extern "C" {
void VTPrintInit(BOOL MC)
{
  TVTPrnVar vpv;
  BOOL Sel;

  Sel = ! MC && Select;
  if (! PrnBox(HVTWin,&Sel)) return;

  /* set print region */
  if (! MC && Sel)
  {
    /* print selected region */
    vpv.PrintStart = SelectStart;
    vpv.PrintEnd = SelectEnd;
  }
  else if (MC && ! PrintEX)
  {
    /* print scroll region */
    vpv.PrintStart.x = 0;
    vpv.PrintStart.y = PageStart + CursorTop;
    vpv.PrintEnd.x = NumOfColumns;
    vpv.PrintEnd.y = PageStart + CursorBottom;
  }
  else {
    /* print current screen */
    vpv.PrintStart.x = 0;
    vpv.PrintStart.y = PageStart;
    vpv.PrintEnd.x = NumOfColumns;
    vpv.PrintEnd.y = PageStart + NumOfLines - 1;
  }
  if (vpv.PrintEnd.y > BuffEnd-1)
    vpv.PrintEnd.y = BuffEnd-1;

  /* start printing */
  if (! PrnStart(ts.Title)) return;
  if (LoadTTFILE())
  {
    LockBuffer();

    vpv.HWin = HVTWin;
    vpv.PrintDC = PrintDC;
    vpv.ts = &ts;
    vpv.NumOfColumns = NumOfColumns;
    vpv.BuffStartAbs = BuffStartAbs;
    vpv.BufferSize = BufferSize;
    vpv.CodeBuff = CodeBuff;
    vpv.AttrBuff = AttrBuff;
    vpv.AttrBuff2 = AttrBuff2;

    (*VTPrint)(&vpv);
    UnlockBuffer();

    FreeTTFILE();
  }

  PrnStop();
}
}

/* print abort dialog */
BOOL CPrnAbortDlg::Create()
{
  BOOL Ok;
  CWnd* pParent;

  if (ActiveWin==IdVT)
    pParent = (CWnd*)pVTWin;
  else
    pParent = (CWnd*)pTEKWin;

  Ok = (CDialog::Create(CPrnAbortDlg::IDD, pParent));
  HPrnAbortDlg = GetSafeHwnd();
  return Ok;
}

BOOL CPrnAbortDlg::OnCommand(WPARAM wParam, LPARAM lParam)
{
  PrintAbort = TRUE;
  return (CDialog::OnCommand(wParam,lParam));
}

/* printer emulation routines */
extern "C" {
void OpenPrnFile()
{
  if (HPrnFile > 0) return;
  HPrnFile = _lopen(PrnFName,OF_WRITE);
  if (HPrnFile <= 0)
    HPrnFile = _lcreat(PrnFName,0);
  if (HPrnFile > 0)
    _llseek(HPrnFile,0,2);
}
}

extern "C" {
void ClosePrnFile()
{
  if (HPrnFile > 0)
    _lclose(HPrnFile);
  HPrnFile = 0;
}
}

void WriteToPrnFile(BYTE b)
{
  switch (b) {
    case NUL: break;
    case DC1: break;
    case DC3: break;
    default:
      _lwrite(HPrnFile,(LPSTR)&b,1);
  }
}

extern "C" {
void PrintCurrentLine(BYTE b)
{
  int i;

  if (HPrnFile<=0) return;
  i = NumOfColumns;
  while ((i>0) && (CodeLine[i-1]==0x20))
    i--;
  _lwrite(HPrnFile,CodeLine,i);

  if ((b>=LF) && (b<=FF))
  {
    _lwrite(HPrnFile,"\015",1);
    _lwrite(HPrnFile,(LPCSTR)&b,1);
  }
}
}

extern "C" {
void PrnParse(BYTE b)
{
  if (HPrnFile<=0)
  {
    PrinterMode = FALSE;
    return;
  }

  switch (PrnParseMode) {
    case IdPrnNormal:
      switch (b) {
        case ESC: PrnParseMode = IdPrnESC; break;
        case CSI:
          DB_CSI = FALSE;
          PrnParseMode = IdPrnCSI;
          break;
        default:
          WriteToPrnFile(b);
      }
      break;
    case IdPrnESC:
      if (b=='[')
      {
        DB_CSI = TRUE;
        PrnParseMode = IdPrnCSI;
      }
      else {
        WriteToPrnFile(ESC);
        PrnParseMode = IdPrnNormal;
        PrnParse(b);
      }
      break;
    case IdPrnCSI:
      if ((b==0x34) || (b==0x35))
      {
        PCM = b;
        PrnParseMode = IdPrnCSIi;
      }
      else {
        if (DB_CSI)
        {
          WriteToPrnFile(ESC);
          WriteToPrnFile('[');
        }
        else
          WriteToPrnFile(CSI);
        PrnParseMode = IdPrnNormal;
        PrnParse(b);
      }
      break;
    case IdPrnCSIi:
      if (b=='i')
      {
        PrinterMode = PCM==0x35;
        if ((! PrinterMode) &&
            (! AutoPrintMode))
          ClosePrnFile();
        PrnParseMode = IdPrnNormal;
      }
      else {
        if (DB_CSI)
        {
          WriteToPrnFile(ESC);
          WriteToPrnFile('[');
        }
        else
          WriteToPrnFile(CSI);
        WriteToPrnFile(PCM);
        PrnParseMode = IdPrnNormal;
        PrnParse(b);
      }
      break;
    default:
      PrnParseMode = IdPrnNormal;
      PrnParse(b);
  }
}
}

