/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TTDLG.DLL, dialog boxes */
#include "teraterm.h"
#include <stdio.h>
#include <io.h>
#include <direct.h>
#include "tttypes.h"
#include "ttlib.h"
#include "dlglib.h"
#include "ttcommon.h"

#define Id_TermDialogJP 100
#define Id_TermDialog 1100
#define Id_TermWidth 101
#define Id_TermHeight 102
#define Id_TermIsWin 103
#define Id_TermResize 104
#define Id_TermCRRcv 105
#define Id_TermCRLFRcv 106
#define Id_TermCRSend 107
#define Id_TermCRLFSend 108
#define Id_TermLocalEcho 109
#define Id_TermAnsbackText 110
#define Id_TermAnsback 111
#define Id_TermSJIS 112
#define Id_TermEUC 113
#define Id_TermJIS 114
#define Id_TermKana 115
#define Id_TermSJISSend 116
#define Id_TermEUCSend 117
#define Id_TermJISSend 118
#define Id_TermKanaSend 119
#define Id_TermKIN0 120
#define Id_TermKIN1 121
#define Id_TermKIN2 122
#define Id_TermKOut0 123
#define Id_TermKOut1 124
#define Id_TermKOut3 126
#define Id_TermAutoWinSwitch 127
#define Id_TermVT100 128
#define Id_TermVT382 136
#define Id_TermHelp 199

#define Id_WinDialog 200
#define Id_WinTitle 201
#define Id_WinBlock 202
#define Id_WinVert 203
#define Id_WinHorz 204
#define Id_WinHideMenu 205
#define Id_Win3D 206
#define Id_WinColorEmu 207
#define Id_WinScroll1 208
#define Id_WinScroll2 209
#define Id_WinScroll3 210
#define Id_WinText 211
#define Id_WinBack 212
#define Id_WinRev 213
#define Id_WinRed 214
#define Id_WinGreen 215
#define Id_WinBlue 216
#define Id_WinRedBar 217
#define Id_WinGreenBar 218
#define Id_WinBlueBar 219
#define Id_WinAttrText 220
#define Id_WinAttr 221
#define Id_WinHelp 299

#define Id_KeybDialogJP 300
#define Id_KeybDialog 1200
#define Id_KeybBS 301
#define Id_KeybDEL 302
#define Id_KeybIME 303
#define Id_KeybEnable 304
#define Id_KeybInline 305
#define Id_KeybMeta 306
#define Id_KeybHelp 399

#define Id_SerialDialog 400
#define Id_SerialPort 401
#define Id_SerialBaud 402
#define Id_SerialData 403
#define Id_SerialParity 404
#define Id_SerialStop 405
#define Id_SerialFlow 406
#define Id_SerialDelayChar 407
#define Id_SerialDelayLine 408
#define Id_SerialHelp 499

#define Id_TCPIPDialog 500
#define Id_TCPIPHost 501
#define Id_TCPIPAdd 502
#define Id_TCPIPList 503
#define Id_TCPIPDelete 504
#define Id_TCPIPAutoClose 505
#define Id_TCPIPPort 506
#define Id_TCPIPTelnet 507
#define Id_TCPIPTermTypeLabel 508
#define Id_TCPIPTermType 509
#define Id_TCPIPHelp 599

#define Id_HostDialog 600
#define Id_HostTCPIP 601
#define Id_HostSerial 602
#define Id_HostNameLabel 603
#define Id_HostName 604
#define Id_HostTelnet 605
#define Id_HostTCPPortLabel 606
#define Id_HostTCPPort 607
#define Id_HostComLabel 608
#define Id_HostCom 609
#define Id_HostHelp 699

#define Id_DirDialog 700
#define Id_DirCurrent 701
#define Id_DirNew 702
#define Id_DirHelp 799

#define Id_AboutDialog 800
#define Id_AboutMode 801
#define Id_AboutMem 802
#define Id_AboutRes 803

#define Id_FontDlg 900
#define Id_FontBold 901

#define Id_GenDialog 1000
#define Id_GenTCP 1001
#define Id_GenSerial 1002
#define Id_GenEnglish 1003
#define Id_GenJapanese 1004
#define Id_GenHelp 1099

HANDLE HInst;

BOOL CALLBACK TermDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PTTSet ts;
  WORD w;
  char Temp[81];

  switch (Message) {
    case WM_INITDIALOG:
        ts = (PTTSet)lParam;
        SetWindowLong(Dialog, DWL_USER, lParam);
        if ( ! ts->Ok ) /* English mode */
          /* Japanese terminal -> English terminal */
          switch (ts->TerminalID) {
            case IdVT100J: ts->TerminalID = IdVT100; break;
            case IdVT102J: ts->TerminalID = IdVT102; break;
            case IdVT220J: ts->TerminalID = IdVT282; break;
          }

        SetDlgItemInt(Dialog,Id_TermWidth,ts->TerminalWidth,FALSE);
        SendDlgItemMessage(Dialog, Id_TermWidth, EM_LIMITTEXT,3, 0);

        SetDlgItemInt(Dialog,Id_TermHeight,ts->TerminalHeight,FALSE);
        SendDlgItemMessage(Dialog, Id_TermHeight, EM_LIMITTEXT,3, 0);

        SetRB(Dialog,ts->TermIsWin,Id_TermIsWin,Id_TermIsWin);
        SetRB(Dialog,ts->AutoWinResize,Id_TermResize,Id_TermResize);
        if ( ts->TermIsWin>0 )
          DisableDlgItem(Dialog,Id_TermResize,Id_TermResize);

        SetRB(Dialog,ts->CRReceive,Id_TermCRRcv,Id_TermCRLFRcv);

        SetRB(Dialog,ts->CRSend,Id_TermCRSend,Id_TermCRLFSend);

        if ( ts->BPAuto )
          DisableDlgItem(Dialog,Id_TermAnsbackText,Id_TermAnsback);
        else {
          Str2Hex(ts->Answerback,Temp,ts->AnswerbackLen,
                  sizeof(Temp)-1,FALSE);
          SetDlgItemText(Dialog, Id_TermAnsback, Temp);
          SendDlgItemMessage(Dialog, Id_TermAnsback, EM_LIMITTEXT,
                             sizeof(Temp) - 1, 0);
        }

        SetRB(Dialog,ts->LocalEcho,Id_TermLocalEcho,Id_TermLocalEcho);

        SetRB(Dialog,ts->KanjiCode,Id_TermSJIS,Id_TermJIS);

        if ( ts->KanjiCode!=IdJIS )
          DisableDlgItem(Dialog,Id_TermKana,Id_TermKana);

        SetRB(Dialog,ts->JIS7Katakana,Id_TermKana,Id_TermKana);

        SetRB(Dialog,ts->KanjiCodeSend,Id_TermSJISSend,Id_TermJISSend);

        if ( ts->KanjiCodeSend!=IdJIS )
          DisableDlgItem(Dialog,Id_TermKanaSend,Id_TermKOut3);

        SetRB(Dialog,ts->JIS7KatakanaSend,Id_TermKanaSend,Id_TermKanaSend);

        SetRB(Dialog,ts->KanjiIn,Id_TermKIN1,Id_TermKIN2);

        if ( ! ts->AllowWrongSequence )
          DisableDlgItem(Dialog,Id_TermKOut3,Id_TermKOut3);

        SetRB(Dialog,ts->KanjiOut,Id_TermKOut1,Id_TermKOut3);

        SetRB(Dialog,ts->AutoWinSwitch,Id_TermAutoWinSwitch,Id_TermAutoWinSwitch);

        SetRB(Dialog,ts->TerminalID,Id_TermVT100,Id_TermVT382);

        return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);

          if ( ts!=NULL )
          {
            ts->Ok = TRUE;

            ts->TerminalWidth = GetDlgItemInt(Dialog,Id_TermWidth,NULL,FALSE);
            if ( ts->TerminalWidth<1 ) ts->TerminalWidth = 1;
            if ( ts->TerminalWidth>500 ) ts->TerminalWidth = 500;

            ts->TerminalHeight = GetDlgItemInt(Dialog,Id_TermHeight,NULL,FALSE);
            if ( ts->TerminalHeight<1 ) ts->TerminalHeight = 1;
            if ( ts->TerminalHeight>500 ) ts->TerminalHeight = 500;

            GetRB(Dialog,&ts->TermIsWin,Id_TermIsWin,Id_TermIsWin);
            GetRB(Dialog,&ts->AutoWinResize,Id_TermResize,Id_TermResize);

            GetRB(Dialog,&ts->CRReceive,Id_TermCRRcv,Id_TermCRLFRcv);

            GetRB(Dialog,&ts->CRSend,Id_TermCRSend,Id_TermCRLFSend);

            if ( ! ts->BPAuto )
            {
              GetDlgItemText(Dialog, Id_TermAnsback,Temp,sizeof(Temp));
              ts->AnswerbackLen =
                Hex2Str(Temp,ts->Answerback,sizeof(ts->Answerback));
            }

            GetRB(Dialog,&ts->LocalEcho,Id_TermLocalEcho,Id_TermLocalEcho);

            GetRB(Dialog,&ts->KanjiCode,Id_TermSJIS,Id_TermJIS);

            GetRB(Dialog,&ts->JIS7Katakana,Id_TermKana,Id_TermKana);

            GetRB(Dialog,&ts->KanjiCodeSend,Id_TermSJISSend,Id_TermJISSend);

            GetRB(Dialog,&ts->JIS7KatakanaSend,Id_TermKanaSend,Id_TermKanaSend);

            GetRB(Dialog,&ts->KanjiIn,Id_TermKIN1,Id_TermKIN2);

            GetRB(Dialog,&ts->KanjiOut,Id_TermKOut1,Id_TermKOut3);

            GetRB(Dialog,&ts->AutoWinSwitch,Id_TermAutoWinSwitch,Id_TermAutoWinSwitch);

            GetRB(Dialog,&ts->TerminalID,Id_TermVT100,Id_TermVT382);

          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
	  ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
          if ( ts!=NULL )
            ts->Ok = FALSE;
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_TermIsWin:
          GetRB(Dialog,&w,Id_TermIsWin,Id_TermIsWin);
          if ( w==0 )
            EnableDlgItem(Dialog,Id_TermResize,Id_TermResize);
          else
            DisableDlgItem(Dialog,Id_TermResize,Id_TermResize);
          break;

        case Id_TermSJIS:
        case Id_TermEUC:
          DisableDlgItem(Dialog,Id_TermKana,Id_TermKana);
          break;
        case Id_TermJIS:
          EnableDlgItem(Dialog,Id_TermKana,Id_TermKana);
          break;
        case Id_TermSJISSend:
        case Id_TermEUCSend:
          DisableDlgItem(Dialog,Id_TermKanaSend,Id_TermKOut3);
          break;
        case Id_TermJISSend:
          EnableDlgItem(Dialog,Id_TermKanaSend,Id_TermKOut3);
	  ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
          if ( ! ts->AllowWrongSequence )
            DisableDlgItem(Dialog,Id_TermKOut3,Id_TermKOut3);
          break;
        case Id_TermHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
      }
  }
  return FALSE;
}

  void DispSample(HWND Dialog, PTTSet ts, int IAttr)
  {
    int i,x,y;
    COLORREF Text, Back;
    int DX[3];
    TEXTMETRIC Metrics;
    RECT Rect, TestRect;
    int FW,FH;
    HDC DC;

    DC = GetDC(Dialog);
    Text = RGB(ts->TmpColor[IAttr][0],
                ts->TmpColor[IAttr][1],
                ts->TmpColor[IAttr][2]);
    Text = GetNearestColor(DC, Text);
    Back = RGB(ts->TmpColor[IAttr][3],
                ts->TmpColor[IAttr][4],
                ts->TmpColor[IAttr][5]);
    Back = GetNearestColor(DC, Back);
    SetTextColor(DC, Text);
    SetBkColor(DC, Back);
    SelectObject(DC,ts->SampleFont);
    GetTextMetrics(DC, &Metrics);
    FW = Metrics.tmAveCharWidth;
    FH = Metrics.tmHeight;
    for (i = 0 ; i <= 2 ; i++) DX[i] = FW;
    GetClientRect(Dialog,&Rect);
    TestRect.left = Rect.left + (int)((Rect.right-Rect.left)*0.65);
    TestRect.right = Rect.left + (int)((Rect.right-Rect.left)*0.93);
    TestRect.top = Rect.top + (int)((Rect.bottom-Rect.top)*0.54);
    TestRect.bottom = Rect.top + (int)((Rect.bottom-Rect.top)*0.94);
    x = (int)((TestRect.left+TestRect.right) / 2 - FW * 1.5);
    y = (TestRect.top+TestRect.bottom-FH) / 2;
    ExtTextOut(DC, x,y, ETO_CLIPPED | ETO_OPAQUE,
               &TestRect, "ABC", 3, &(DX[0]));
    ReleaseDC(Dialog,DC);
  }

  void ChangeColor(HWND Dialog, PTTSet ts, int IAttr, int IOffset)
  {
    SetDlgItemInt(Dialog,Id_WinRed,ts->TmpColor[IAttr][IOffset],FALSE);
    SetDlgItemInt(Dialog,Id_WinGreen,ts->TmpColor[IAttr][IOffset+1],FALSE);
    SetDlgItemInt(Dialog,Id_WinBlue,ts->TmpColor[IAttr][IOffset+2],FALSE);

    DispSample(Dialog,ts,IAttr);
  }

  void ChangeSB
    (HWND Dialog, PTTSet ts, int IAttr, int IOffset)
  {
    HDC HRed, HGreen, HBlue;

    HRed = GetDlgItem(Dialog, Id_WinRedBar);
    HGreen = GetDlgItem(Dialog, Id_WinGreenBar);
    HBlue = GetDlgItem(Dialog, Id_WinBlueBar);
    
    SetScrollPos(HRed,SB_CTL,ts->TmpColor[IAttr][IOffset+0],TRUE);
    SetScrollPos(HGreen,SB_CTL,ts->TmpColor[IAttr][IOffset+1],TRUE);
    SetScrollPos(HBlue,SB_CTL,ts->TmpColor[IAttr][IOffset+2],TRUE);
    ChangeColor(Dialog,ts,IAttr,IOffset);
  }

  void RestoreVar(HWND Dialog, PTTSet ts, int *IAttr, int *IOffset)
  {
    WORD w;

    GetRB(Dialog,&w,Id_WinText,Id_WinBack);
    if ( w==2 ) *IOffset = 3;
           else *IOffset = 0;
    if ( (ts!=NULL) && ts->VTFlag )
      *IAttr = SendDlgItemMessage(Dialog, Id_WinAttr, CB_GETCURSEL,
                                  0,0);
    else
      *IAttr = 0;
  }

BOOL CALLBACK WinDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PTTSet ts;
  HWND Wnd, HRed, HGreen, HBlue;
  int IAttr, IOffset;
  WORD i, pos;
  HDC DC;

  switch (Message) {
    case WM_INITDIALOG:
      ts = (PTTSet)lParam;
      SetWindowLong(Dialog, DWL_USER, lParam);

      ts->Ok = FALSE;

      SetDlgItemText(Dialog, Id_WinTitle, ts->Title);
      SendDlgItemMessage(Dialog, Id_WinTitle, EM_LIMITTEXT,
                           sizeof(ts->Title)-1, 0);

      SetRB(Dialog,ts->PopupMenu,Id_WinHideMenu,Id_WinHideMenu);
      if ( ts->HideTitle==0 )
        EnableDlgItem(Dialog,Id_WinHideMenu,Id_WinHideMenu);         

      if ( ts->VTFlag )
      {
        ShowDlgItem(Dialog,Id_WinScroll1,Id_WinScroll3);
        SetRB(Dialog,ts->EnableScrollBuff,Id_WinScroll1,Id_WinScroll1);
        SetDlgItemInt(Dialog,Id_WinScroll2,ts->ScrollBuffMax,FALSE);
        SendDlgItemMessage(Dialog, Id_WinScroll2, EM_LIMITTEXT,5, 0);
        if ( ts->EnableScrollBuff==0 )
          DisableDlgItem(Dialog,Id_WinScroll2,Id_WinScroll3);
        for (i = 0 ; i <= 1 ; i++)
        {
          ts->TmpColor[0][i*3]   = GetRValue(ts->VTColor[i]);
          ts->TmpColor[0][i*3+1] = GetGValue(ts->VTColor[i]);
          ts->TmpColor[0][i*3+2] = GetBValue(ts->VTColor[i]);
          ts->TmpColor[1][i*3]   = GetRValue(ts->VTBoldColor[i]);
          ts->TmpColor[1][i*3+1] = GetGValue(ts->VTBoldColor[i]);
          ts->TmpColor[1][i*3+2] = GetBValue(ts->VTBoldColor[i]);
          ts->TmpColor[2][i*3]   = GetRValue(ts->VTBlinkColor[i]);
          ts->TmpColor[2][i*3+1] = GetGValue(ts->VTBlinkColor[i]);
          ts->TmpColor[2][i*3+2] = GetBValue(ts->VTBlinkColor[i]);
        }
        ShowDlgItem(Dialog,Id_WinAttrText,Id_WinAttr);
        SendDlgItemMessage(Dialog, Id_WinAttr, CB_ADDSTRING,
                           0, (LONG)"Normal");
        SendDlgItemMessage(Dialog, Id_WinAttr, CB_ADDSTRING,
                           0, (LONG)"Bold");
        SendDlgItemMessage(Dialog, Id_WinAttr, CB_ADDSTRING,
                           0, (LONG)"Blink");
        SendDlgItemMessage(Dialog, Id_WinAttr, CB_SETCURSEL,
                           0,0);
      }
      else {
        for (i = 0 ; i <=1 ; i++)
        {
          ts->TmpColor[0][i*3]   = GetRValue(ts->TEKColor[i]);
          ts->TmpColor[0][i*3+1] = GetGValue(ts->TEKColor[i]);
          ts->TmpColor[0][i*3+2] = GetBValue(ts->TEKColor[i]);
        }
        ShowDlgItem(Dialog,Id_WinColorEmu,Id_WinColorEmu);
        SetRB(Dialog,ts->TEKColorEmu,Id_WinColorEmu,Id_WinColorEmu);
      }
      SetRB(Dialog,1,Id_WinText,Id_WinBack);

      SetRB(Dialog,ts->CursorShape,Id_WinBlock,Id_WinHorz);

      IAttr = 0;
      IOffset = 0;

      HRed = GetDlgItem(Dialog, Id_WinRedBar);
      SetScrollRange(HRed,SB_CTL,0,255,TRUE);

      HGreen = GetDlgItem(Dialog, Id_WinGreenBar);
      SetScrollRange(HGreen,SB_CTL,0,255,TRUE);

      HBlue = GetDlgItem(Dialog, Id_WinBlueBar);
      SetScrollRange(HBlue,SB_CTL,0,255,TRUE);

      ChangeSB(Dialog,ts,IAttr,IOffset);

      return TRUE;

    case WM_COMMAND:
      ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
      RestoreVar(Dialog,ts,&IAttr,&IOffset);
      switch (LOWORD(wParam)) {
        case IDOK:
          if ( ts!=NULL )
          {
            ts->Ok = TRUE;

            GetDlgItemText(Dialog,Id_WinTitle,ts->Title,sizeof(ts->Title));
            if ( ts->HideTitle==0 )
              GetRB(Dialog,&ts->PopupMenu,Id_WinHideMenu,Id_WinHideMenu);

            DC = GetDC(Dialog);
            if ( ts->VTFlag )
            {
              GetRB(Dialog,&ts->EnableScrollBuff,Id_WinScroll1,Id_WinScroll1);
              if ( ts->EnableScrollBuff>0 )
              {
                ts->ScrollBuffMax =
                  GetDlgItemInt(Dialog,Id_WinScroll2,NULL,FALSE);
                if ( ts->ScrollBuffMax>10000 )
                  ts->ScrollBuffMax = 10000;
              }
              for (i = 0 ; i <= 1 ; i++)
              {
                ts->VTColor[i] =
                  RGB(ts->TmpColor[0][i*3],
                      ts->TmpColor[0][i*3+1],
                      ts->TmpColor[0][i*3+2]);
                ts->VTBoldColor[i] =
                  RGB(ts->TmpColor[1][i*3],
                      ts->TmpColor[1][i*3+1],
                      ts->TmpColor[1][i*3+2]);
                 ts->VTBlinkColor[i] =
                  RGB(ts->TmpColor[2][i*3],
                      ts->TmpColor[2][i*3+1],
                      ts->TmpColor[2][i*3+2]);
                 ts->VTColor[i] = GetNearestColor(DC,ts->VTColor[i]);
                 ts->VTBoldColor[i] = GetNearestColor(DC,ts->VTBoldColor[i]);
                 ts->VTBlinkColor[i] = GetNearestColor(DC,ts->VTBlinkColor[i]);
              }
            }
            else {
              for (i = 0 ; i <= 1 ; i++)
              {
                ts->TEKColor[i] =
                  RGB(ts->TmpColor[0][i*3],
                      ts->TmpColor[0][i*3+1],
                      ts->TmpColor[0][i*3+2]);
                ts->TEKColor[i] = GetNearestColor(DC,ts->TEKColor[i]);
              }
              GetRB(Dialog,&ts->TEKColorEmu,Id_WinColorEmu,Id_WinColorEmu);
            }
            ReleaseDC(Dialog,DC);

            GetRB(Dialog,&ts->CursorShape,Id_WinBlock,Id_WinHorz);
            
          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_WinScroll1:
          if ( ts==NULL ) return TRUE;
          GetRB(Dialog,&i,Id_WinScroll1,Id_WinScroll1);
          if ( i>0 )
            EnableDlgItem(Dialog,Id_WinScroll2,Id_WinScroll3);
          else
            DisableDlgItem(Dialog,Id_WinScroll2,Id_WinScroll3);
          break;

        case Id_WinText:
          if ( ts==NULL ) return TRUE;
          IOffset = 0;
          ChangeSB(Dialog,ts,IAttr,IOffset);
          break;
          
        case Id_WinBack:
          if ( ts==NULL ) return TRUE;
          IOffset = 3;
          ChangeSB(Dialog,ts,IAttr,IOffset);
          break;

        case Id_WinRev:
          if ( ts==NULL ) return TRUE;          
          i = ts->TmpColor[IAttr][0];
          ts->TmpColor[IAttr][0] = ts->TmpColor[IAttr][3];
          ts->TmpColor[IAttr][3] = i;
          i = ts->TmpColor[IAttr][1];
          ts->TmpColor[IAttr][1] = ts->TmpColor[IAttr][4];
          ts->TmpColor[IAttr][4] = i;
          i = ts->TmpColor[IAttr][2];
          ts->TmpColor[IAttr][2] = ts->TmpColor[IAttr][5];
          ts->TmpColor[IAttr][5] = i;

          ChangeSB(Dialog,ts,IAttr,IOffset);
          break;
          
        case Id_WinAttr:
          if ( ts!=NULL ) ChangeSB(Dialog,ts,IAttr,IOffset);
          break;

        case Id_WinHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
          break;
      }
      break;

    case WM_PAINT:
      ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
      if ( ts==NULL ) return TRUE;
      RestoreVar(Dialog,ts,&IAttr,&IOffset);
      DispSample(Dialog,ts,IAttr);
      break;

    case WM_HSCROLL:
      ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
      if ( ts==NULL ) return TRUE;
      RestoreVar(Dialog,ts,&IAttr,&IOffset);
      HRed = GetDlgItem(Dialog, Id_WinRedBar);
      HGreen = GetDlgItem(Dialog, Id_WinGreenBar);
      HBlue = GetDlgItem(Dialog, Id_WinBlueBar);

      Wnd = (HWND)lParam;
      if ( Wnd == HRed ) i = IOffset;
      else if ( Wnd == HGreen ) i = IOffset + 1;
      else if ( Wnd == HBlue ) i = IOffset + 2;
      else return TRUE;
      pos = ts->TmpColor[IAttr][i];
      switch (LOWORD(wParam)) {
        case SB_BOTTOM:
          pos = 255;
          break;
        case SB_LINEDOWN:
          if (pos<255) pos++;
          break;
        case SB_LINEUP:
          if (pos>0) pos--;
          break;
        case SB_PAGEDOWN:
          pos = pos + 16;
          break;
        case SB_PAGEUP:
          pos = pos - 16;
          break;
        case SB_THUMBPOSITION:
          pos = HIWORD(wParam);
          break;
        case SB_THUMBTRACK:
          pos = HIWORD(wParam);
          break;
        case SB_TOP:
          pos = 0;
          break;
        default:
          return TRUE;
      }
      if (pos > 255) pos = 255;
      ts->TmpColor[IAttr][i] = pos;
      SetScrollPos(Wnd,SB_CTL,pos,TRUE);
      ChangeColor(Dialog,ts,IAttr,IOffset);
      return FALSE;
  }
  return FALSE;
}

BOOL CALLBACK KeybDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PTTSet ts;
  WORD w;

  switch (Message) {
    case WM_INITDIALOG:
      ts = (PTTSet)lParam;
      SetWindowLong(Dialog, DWL_USER, lParam);

      SetRB(Dialog,ts->BSKey,Id_KeybBS,Id_KeybDEL);

      SetRB(Dialog,ts->UseIME,Id_KeybEnable,Id_KeybEnable);
      SetRB(Dialog,ts->IMEInline,Id_KeybInline,Id_KeybInline);
      if ( ts->UseIME==0 )
        DisableDlgItem(Dialog,Id_KeybInline,Id_KeybInline);
      SetRB(Dialog,ts->MetaKey,Id_KeybMeta,Id_KeybMeta);
      return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
          if ( ts!=NULL )
          {
            ts->Ok = TRUE;

            GetRB(Dialog,&ts->BSKey,Id_KeybBS,Id_KeybDEL);

            GetRB(Dialog,&ts->UseIME,Id_KeybEnable,Id_KeybEnable);

            GetRB(Dialog,&ts->IMEInline,Id_KeybInline,Id_KeybInline);

            GetRB(Dialog,&ts->MetaKey,Id_KeybMeta,Id_KeybMeta);

          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
	  ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
          if ( ts!=NULL )
            ts->Ok = FALSE;
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_KeybEnable:
          GetRB(Dialog,&w,Id_KeybEnable,Id_KeybEnable);
          if ( w==0 )
            DisableDlgItem(Dialog,Id_KeybInline,Id_KeybInline);
          else
            EnableDlgItem(Dialog,Id_KeybInline,Id_KeybInline);
          break;

        case Id_KeybHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
      }
  }
  return FALSE;
}

static PCHAR PortList[] = {"COM1","COM2","COM3","COM4",NULL};
static PCHAR BaudList[] =
  {"110","300","600","1200","2400","4800","9600",
   "19200","38400","57600","115200",NULL};
static PCHAR DataList[] = {"7 bit","8 bit",NULL};
static PCHAR ParityList[] = {"even","odd","none",NULL};
static PCHAR StopList[] = {"1 bit","2 bit",NULL};
static PCHAR FlowList[] = {"Xon/Xoff","hardware","none",NULL};

BOOL CALLBACK SerialDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PTTSet ts;

  switch (Message) {
    case WM_INITDIALOG:
      ts = (PTTSet)lParam;
      ts->Ok = FALSE;
      SetWindowLong(Dialog, DWL_USER, lParam);

      SetDropDownList(Dialog, Id_SerialPort, PortList, ts->ComPort);
      SetDropDownList(Dialog, Id_SerialBaud, BaudList, ts->Baud);
      SetDropDownList(Dialog, Id_SerialData, DataList, ts->DataBit);
      SetDropDownList(Dialog, Id_SerialParity, ParityList, ts->Parity);
      SetDropDownList(Dialog, Id_SerialStop, StopList, ts->StopBit);
      SetDropDownList(Dialog, Id_SerialFlow, FlowList, ts->Flow);

      SetDlgItemInt(Dialog,Id_SerialDelayChar,ts->DelayPerChar,FALSE);
      SendDlgItemMessage(Dialog, Id_SerialDelayChar, EM_LIMITTEXT,4, 0);

      SetDlgItemInt(Dialog,Id_SerialDelayLine,ts->DelayPerLine,FALSE);
      SendDlgItemMessage(Dialog, Id_SerialDelayLine, EM_LIMITTEXT,4, 0);

      return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
          if ( ts!=NULL )
          {
            ts->Ok = TRUE;

	    ts->ComPort = GetCurSel(Dialog, Id_SerialPort);
	    ts->Baud = GetCurSel(Dialog, Id_SerialBaud);
	    ts->DataBit = GetCurSel(Dialog, Id_SerialData);
	    ts->Parity = GetCurSel(Dialog, Id_SerialParity);
	    ts->StopBit = GetCurSel(Dialog, Id_SerialStop);
	    ts->Flow = GetCurSel(Dialog, Id_SerialFlow);

            ts->DelayPerChar = GetDlgItemInt(Dialog,Id_SerialDelayChar,NULL,FALSE);

            ts->DelayPerLine = GetDlgItemInt(Dialog,Id_SerialDelayLine,NULL,FALSE);

            ts->PortType = IdSerial;
          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_SerialHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
      }
  }
  return FALSE;
}

BOOL CALLBACK TCPIPDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PTTSet ts;
  char EntName[7];
  char TempHost[HostNameMaxLength+1];
  WORD i, Index;
  HWND HControl;
  BOOL Ok;

  switch (Message) {
    case WM_INITDIALOG:
      ts = (PTTSet)lParam;
      ts->Ok = FALSE;
      SetWindowLong(Dialog, DWL_USER, lParam);

      SendDlgItemMessage(Dialog, Id_TCPIPHost, EM_LIMITTEXT,
                         HostNameMaxLength-1, 0);

      strcpy(EntName,"Host");

      i = 1;
      do {
        if ( i < 10 )
        {
          EntName[4] = (char)(i+0x30);
          EntName[5] = 0;
        }
        else {
          EntName[4] = (char)(i / 10 + 0x30);
          EntName[5] = (char)(i % 10 + 0x30);
          EntName[6] = 0;
        }

        GetPrivateProfileString("Hosts",EntName,"",
                                TempHost,sizeof(TempHost),ts->SetupFName);
        if ( strlen(TempHost) > 0 )
          SendDlgItemMessage(Dialog, Id_TCPIPList, LB_ADDSTRING,
                             0, (LPARAM)TempHost);
        i++;
      } while ((i <= 99) && (strlen(TempHost)>0));

      SetRB(Dialog,ts->AutoWinClose,Id_TCPIPAutoClose,Id_TCPIPAutoClose);

      SetDlgItemInt(Dialog,Id_TCPIPPort,ts->TCPPort,FALSE);

      SetRB(Dialog,ts->Telnet,Id_TCPIPTelnet,Id_TCPIPTelnet);

      SetDlgItemText(Dialog, Id_TCPIPTermType, ts->TermType);
      SendDlgItemMessage(Dialog, Id_TCPIPTermType, EM_LIMITTEXT,
                         sizeof(ts->TermType)-1, 0);

      if ( ts->Telnet==0 )
        DisableDlgItem(Dialog,Id_TCPIPTermTypeLabel,Id_TCPIPTermType);

      return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  ts = (PTTSet)GetWindowLong(Dialog,DWL_USER);
          if ( ts!=NULL )
          {
            ts->Ok = TRUE;

            WritePrivateProfileString("Hosts",NULL,NULL,ts->SetupFName);

            HControl = GetDlgItem(Dialog, Id_TCPIPList);
            Index = SendMessage(HControl, LB_GETCOUNT, 0, 0);
            if ( Index==LB_ERR ) Index = 0;
            if ( Index>99 ) Index = 99;

            strcpy(EntName,"Host");
            for (i = 1 ; i <= Index ; i++)
            {
              SendDlgItemMessage(Dialog, Id_TCPIPList, LB_GETTEXT,
                                 i-1, (LPARAM)TempHost);

              if ( i < 10 )
              {
                EntName[4] = (char)(i+0x30);
                EntName[5] = 0;
              }
              else {
                EntName[4] = (char)(i / 10 + 0x30);
                EntName[5] = (char)(i % 10 + 0x30);
                EntName[6] = 0;
              }
              WritePrivateProfileString("Hosts",EntName,TempHost,ts->SetupFName);
            }

            GetRB(Dialog,&ts->AutoWinClose,Id_TCPIPAutoClose,Id_TCPIPAutoClose);

            ts->TCPPort = GetDlgItemInt(Dialog,Id_TCPIPPort,&Ok,FALSE);
            if ( ! Ok ) ts->TCPPort = 23;

            GetRB(Dialog,&ts->Telnet,Id_TCPIPTelnet,Id_TCPIPTelnet);

            GetDlgItemText(Dialog, Id_TCPIPTermType, ts->TermType,
                         sizeof(ts->TermType));

          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_TCPIPHost:
          if ( HIWORD(wParam)==EN_CHANGE )
          {
            HControl = (HWND)lParam;
            GetDlgItemText(Dialog, Id_TCPIPHost, TempHost, sizeof(TempHost));
            if ( strlen(TempHost)==0 )
              DisableDlgItem(Dialog,Id_TCPIPAdd,Id_TCPIPAdd);
            else
              EnableDlgItem(Dialog,Id_TCPIPAdd,Id_TCPIPAdd);
          }
          break;

        case Id_TCPIPAdd:
          GetDlgItemText(Dialog, Id_TCPIPHost, TempHost, sizeof(TempHost));
          if ( strlen(TempHost)>0 )
          {
            HControl = GetDlgItem(Dialog, Id_TCPIPList);
            Index = SendMessage(HControl, LB_GETCURSEL, 0, 0);
            if ( Index==LB_ERR ) Index = 0;

            SendDlgItemMessage(Dialog, Id_TCPIPList, LB_INSERTSTRING,
                               Index, (LPARAM)TempHost);
            
            SetDlgItemText(Dialog, Id_TCPIPHost, 0);
          }
          break;

        case Id_TCPIPList:
          if ( HIWORD(wParam)==LBN_SELCHANGE )
            EnableDlgItem(Dialog,Id_TCPIPDelete,Id_TCPIPDelete);
          break;

        case Id_TCPIPDelete:
          HControl = GetDlgItem(Dialog, Id_TCPIPList);
          Index = SendMessage(HControl, LB_GETCURSEL, 0, 0);
          if ( Index==LB_ERR ) return TRUE;
          SendDlgItemMessage(Dialog, Id_TCPIPList, LB_GETTEXT,
                             Index, (LPARAM)TempHost);
          SendDlgItemMessage(Dialog, Id_TCPIPList, LB_DELETESTRING,
                             Index, 0);
          SetDlgItemText(Dialog, Id_TCPIPHost, TempHost);
          DisableDlgItem(Dialog,Id_TCPIPDelete,Id_TCPIPDelete);
          break;

        case Id_TCPIPTelnet:
          GetRB(Dialog,&i,Id_TCPIPTelnet,Id_TCPIPTelnet);
          if ( i==1 )
          {
            EnableDlgItem(Dialog,Id_TCPIPTermTypeLabel,Id_TCPIPTermType);
            SetDlgItemInt(Dialog,Id_TCPIPPort,23,FALSE);
          }
          else
            DisableDlgItem(Dialog,Id_TCPIPTermTypeLabel,Id_TCPIPTermType);
          break;

        case Id_TCPIPHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
          break;
      }
  }
  return FALSE;
}

BOOL CALLBACK HostDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PGetHNRec GetHNRec;
  char EntName[7];
  char TempHost[HostNameMaxLength+1];
  WORD i;
  BOOL Ok;

  switch (Message) {
    case WM_INITDIALOG:
      GetHNRec = (PGetHNRec)lParam;
      SetWindowLong(Dialog, DWL_USER, lParam);

      if ( GetHNRec->PortType==IdFile )
        GetHNRec->PortType = IdTCPIP;
      SetRB(Dialog,GetHNRec->PortType,Id_HostTCPIP,Id_HostSerial);

      strcpy(EntName,"Host");

      i = 1;
      do {
        if ( i < 10 )
        {
          EntName[4] = (char)(i+0x30);
          EntName[5] = 0;
        }
        else {
          EntName[4] = (char)(i / 10 + 0x30);
          EntName[5] = (char)(i % 10 + 0x30);
          EntName[6] = 0;
        }

        GetPrivateProfileString("Hosts",EntName,"",
                                TempHost,sizeof(TempHost),GetHNRec->SetupFN);
        if ( strlen(TempHost) > 0 )
          SendDlgItemMessage(Dialog, Id_HostName, CB_ADDSTRING,
                             0, (LPARAM)TempHost);
        i++;
      } while ((i <= 99) && (strlen(TempHost)>0));

      SendDlgItemMessage(Dialog, Id_HostName, EM_LIMITTEXT,
                         HostNameMaxLength-1, 0);

      SendDlgItemMessage(Dialog, Id_HostName, CB_SETCURSEL,0,0);

      SendDlgItemMessage(Dialog, Id_HostTCPPort, EM_LIMITTEXT,5,0);
      SetDlgItemInt(Dialog,Id_HostTCPPort,GetHNRec->TCPPort,FALSE);

      if ( (GetCOMFlag() & 1) == 0 )
        SendDlgItemMessage(Dialog, Id_HostCom, CB_ADDSTRING,
                           0, (LONG)"COM1");
      if ( (GetCOMFlag() & 2) == 0 )
        SendDlgItemMessage(Dialog, Id_HostCom, CB_ADDSTRING,
                           0, (LONG)"COM2");
      if ( (GetCOMFlag() & 4) == 0 )
        SendDlgItemMessage(Dialog, Id_HostCom, CB_ADDSTRING,
                           0, (LONG)"COM3");
      if ( (GetCOMFlag() & 8) == 0 )
        SendDlgItemMessage(Dialog, Id_HostCom, CB_ADDSTRING,
                           0, (LONG)"COM4");

      SetRB(Dialog,GetHNRec->Telnet,Id_HostTelnet,Id_HostTelnet);

      SendDlgItemMessage(Dialog, Id_HostCom, CB_SETCURSEL,0,0);

      /* All com ports are already used */
      if ( GetCOMFlag() == 15 ) GetHNRec->PortType = IdTCPIP;

      if ( GetHNRec->PortType==IdTCPIP )
        DisableDlgItem(Dialog,Id_HostComLabel,Id_HostCom);
      else
        DisableDlgItem(Dialog,Id_HostNameLabel,Id_HostTCPPort);

      return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  GetHNRec = (PGetHNRec)GetWindowLong(Dialog,DWL_USER);
          if ( GetHNRec!=NULL )
          {
            GetRB(Dialog,&GetHNRec->PortType,Id_HostTCPIP,Id_HostSerial);
            if ( GetHNRec->PortType==IdTCPIP )
              GetDlgItemText(Dialog, Id_HostName, GetHNRec->HostName, HostNameMaxLength);
            else
              GetHNRec->HostName[0] = 0;
            GetRB(Dialog,&GetHNRec->Telnet,Id_HostTelnet,Id_HostTelnet);
            GetHNRec->TCPPort = GetDlgItemInt(Dialog,Id_HostTCPPort,&Ok,FALSE);
            if ( ! Ok ) GetHNRec->TCPPort = 65535;
            memset(EntName,0,sizeof(EntName));
            GetDlgItemText(Dialog, Id_HostCom, EntName, sizeof(EntName)-1);
            GetHNRec->ComPort = (BYTE)(EntName[3])-0x30;
            if ( GetHNRec->ComPort > 4 ) GetHNRec->ComPort = 0;
          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_HostTCPIP:
          EnableDlgItem(Dialog,Id_HostNameLabel,Id_HostTCPPort);
          DisableDlgItem(Dialog,Id_HostComLabel,Id_HostCom);
          return TRUE;

        case Id_HostSerial:
          if ( (GetCOMFlag()!=15) )
            EnableDlgItem(Dialog,Id_HostComLabel,Id_HostCom);
          DisableDlgItem(Dialog,Id_HostNameLabel,Id_HostTCPPort);
          break;

        case Id_HostTelnet:
          GetRB(Dialog,&i,Id_HostTelnet,Id_HostTelnet);
          if ( i==1 )
            SetDlgItemInt(Dialog,Id_HostTCPPort,23,FALSE);
          break;

        case Id_HostHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
      }
  }
  return FALSE;
}

BOOL CALLBACK DirDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PCHAR CurDir;
  char HomeDir[MAXPATHLEN];
  char TmpDir[MAXPATHLEN];

  switch (Message) {
    case WM_INITDIALOG:
      CurDir = (PCHAR)(lParam);
      SetWindowLong(Dialog, DWL_USER, lParam);

      SetDlgItemText(Dialog, Id_DirCurrent, CurDir);
      SendDlgItemMessage(Dialog, Id_DirNew, EM_LIMITTEXT,
                         MAXPATHLEN-1, 0);
      return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  CurDir = (PCHAR)GetWindowLong(Dialog,DWL_USER);
          if ( CurDir!=NULL )
          {
            getcwd(HomeDir,sizeof(HomeDir));
            chdir(CurDir);
            GetDlgItemText(Dialog, Id_DirNew, TmpDir,
                         sizeof(TmpDir));
            if ( strlen(TmpDir)>0 )
            {
              if (chdir(TmpDir) != 0)
              {
                MessageBox(Dialog,"Cannot find directory",
                  "Tera Term: Error",MB_ICONEXCLAMATION);
                chdir(HomeDir);
                return TRUE;
              }
              getcwd(CurDir,MAXPATHLEN);
            }
            chdir(HomeDir);
          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_DirHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
      }
  }
  return FALSE;
}

#pragma off (unreferenced);
BOOL CALLBACK AboutDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
#pragma on (unreferenced);
{
  switch (Message) {
    case WM_INITDIALOG:
      return TRUE;
    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;
      }
  }
  return FALSE;
}

BOOL CALLBACK GenDlg(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PLONG Param;
  WORD L, H;

  switch (Message) {
    case WM_INITDIALOG:
      Param = (PLONG)lParam;
      SetWindowLong(Dialog, DWL_USER, lParam);
      L = LOWORD(*Param);
      /* IdFile -> IdTCPIP */
      if ( L!=IdSerial ) L = IdTCPIP;
      H = HIWORD(*Param);
      SetRB(Dialog,L,Id_GenTCP,Id_GenSerial);
      SetRB(Dialog,H,Id_GenEnglish,Id_GenJapanese);
      return TRUE;

    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  Param = (PLONG)GetWindowLong(Dialog,DWL_USER);
          if ( Param!=NULL )
          {
            GetRB(Dialog,&L,Id_GenTCP,Id_GenSerial);
            GetRB(Dialog,&H,Id_GenEnglish,Id_GenJapanese);
            *Param = MAKELONG(L,H);
          }
          EndDialog(Dialog, 1);
          return TRUE;

        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;

        case Id_GenHelp:
          PostMessage(GetParent(Dialog),WM_USER+MsgDlgHelp2,0,0);
      }
  }
  return FALSE;
}


BOOL FAR PASCAL SetupTerminal(HWND WndParent, PTTSet ts, WORD Language)
{
  ts->Ok = (Language==IdJapanese); /* Japanese flag */
  if ( ts->Ok )
    return
      (BOOL)DialogBoxParam(HInst,
            MAKEINTRESOURCE(Id_TermDialogJP),
            WndParent, TermDlg, (LONG)ts);
  else
    return
      (BOOL)DialogBoxParam(HInst,
            MAKEINTRESOURCE(Id_TermDialog),
            WndParent, TermDlg, (LONG)ts);
}

BOOL FAR PASCAL SetupWin(HWND WndParent, PTTSet ts)
{
  return
    (BOOL)DialogBoxParam(HInst,
          MAKEINTRESOURCE(Id_WinDialog),
          WndParent, WinDlg, (LONG)ts);
}

BOOL FAR PASCAL SetupKeyboard(HWND WndParent, PTTSet ts, WORD Language)
{
  ts->Ok = (Language==IdJapanese); /* Japanese flag */
  if ( ts->Ok )
    return
      (BOOL)DialogBoxParam(HInst,
            MAKEINTRESOURCE(Id_KeybDialogJP),
            WndParent, KeybDlg, (LONG)ts);
  else
    return
      (BOOL)DialogBoxParam(HInst,
            MAKEINTRESOURCE(Id_KeybDialog),
            WndParent, KeybDlg, (LONG)ts);
}

BOOL FAR PASCAL SetupSerialPort(HWND WndParent, PTTSet ts)
{
  return
    (BOOL)DialogBoxParam(HInst,
          MAKEINTRESOURCE(Id_SerialDialog),
          WndParent, SerialDlg, (LONG)ts);
}

BOOL FAR PASCAL SetupTCPIP(HWND WndParent, PTTSet ts)
{
  return
    (BOOL)DialogBoxParam(HInst,
          MAKEINTRESOURCE(Id_TCPIPDialog),
          WndParent, TCPIPDlg, (LONG)ts);
}

BOOL FAR PASCAL GetHostName(HWND WndParent, PGetHNRec GetHNRec)
{
  return
    (BOOL)DialogBoxParam(HInst,
          MAKEINTRESOURCE(Id_HostDialog),
          WndParent, HostDlg, (LONG)GetHNRec);
}

BOOL FAR PASCAL ChangeDirectory(HWND WndParent, PCHAR CurDir)
{
  return
    (BOOL)DialogBoxParam(HInst,
          MAKEINTRESOURCE(Id_DirDialog),
          WndParent, DirDlg, (LONG)CurDir);
}

BOOL FAR PASCAL AboutDialog(HWND WndParent)
{
  return
    (BOOL)DialogBox(HInst,
          MAKEINTRESOURCE(Id_AboutDialog),
          WndParent, AboutDlg);
}

BOOL CALLBACK TFontHook(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  LPCHOOSEFONT cf;
  PWORD pw;

  switch (Message) {
    case WM_INITDIALOG:
      cf = (LPCHOOSEFONT)lParam;
      pw = (PWORD)(cf->lCustData);
      SetWindowLong(Dialog, DWL_USER, (LONG)pw);
      ShowDlgItem(Dialog,Id_FontBold,Id_FontBold);
      SetRB(Dialog, *pw & 1,
            Id_FontBold,Id_FontBold);
      break;
    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  pw = (PWORD)GetWindowLong(Dialog,DWL_USER);
          if ( pw!=NULL )
          {
            GetRB(Dialog,pw,
              Id_FontBold,Id_FontBold);
          }
          break;
        case IDCANCEL:
	  break;
      }
  }
  return FALSE;
}

BOOL FAR PASCAL ChooseFontDlg(HWND WndParent, LPLOGFONT LogFont,PWORD Bold)
{
  CHOOSEFONT cf;

  memset(&cf, 0, sizeof(CHOOSEFONT));
  cf.lStructSize = sizeof(CHOOSEFONT);
  cf.hwndOwner = WndParent;
  cf.lpLogFont = LogFont;
  cf.Flags = CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT |
             CF_FIXEDPITCHONLY | CF_SHOWHELP | CF_ENABLETEMPLATE;
  if ( Bold!=NULL )
  {
    cf.Flags = cf.Flags | CF_ENABLEHOOK;
    cf.lpfnHook = (LPCFHOOKPROC)(&TFontHook);

    cf.lCustData = (DWORD)Bold;
  }
  cf.lpTemplateName = MAKEINTRESOURCE(Id_FontDlg);
  cf.nFontType = REGULAR_FONTTYPE;
  cf.hInstance = HInst;
  return (ChooseFont(&cf));
}

BOOL FAR PASCAL SetupGeneral(HWND WndParent, PWORD Port, PWORD Language)
{
  LONG lParam;

  lParam = MAKELONG(*Port,*Language);
  if ((BOOL)DialogBoxParam(HInst,
        MAKEINTRESOURCE(Id_GenDialog),
        WndParent, (DLGPROC)&GenDlg, (LONG)&lParam))
  {
    *Port = LOWORD(lParam);
    *Language = HIWORD(lParam);
    return TRUE;
  }
  else
    return FALSE;
}

#pragma off (unreferenced);
int APIENTRY LibMain(HANDLE hdll,
                     DWORD  reason,
                     LPVOID reserved)
#pragma on (unreferenced);
{
  HInst = hdll;
  switch( reason ) { 
  case DLL_THREAD_ATTACH:
     /* do thread initialization */
    break;
  case DLL_THREAD_DETACH:
    /* do thread cleanup */
    break;
  case DLL_PROCESS_ATTACH:
     /* do process initialization */
    break;
  case DLL_PROCESS_DETACH:
    /* do process cleanup */
    break;
  }
   return( 1 );
 }
