/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* TTFILE.DLL, file transfer, VT window printing */
#include "teraterm.h"
#include "tttypes.h"
#include <direct.h>
#include "ttftypes.h"
#include "ttlib.h"
#include "dlglib.h"
#include "kermit.h"
#include "xmodem.h"
#include "zmodem.h"
#include "bplus.h"
#include "quickvan.h"

#define Id_GetFnDlg 2000
#define Id_GetFn    2001
#define Id_GetFnHelp 2099

#define Id_FnDlg 2100
#define Id_FnBox 2101
#define Id_FnXCheck 2102
#define Id_FnX1K 2104
#define Id_FnBinary 2105
#define Id_FnAppend 2106
#define Id_FnDlg2 2200
#define Id_Fn2Binary 2201

static HANDLE HInst;

BOOL FAR PASCAL GetSetupFname(HWND HWin, BOOL Save, PTTSet ts)
{
  int i, j;
  OPENFILENAME ofn;
  int Ptr;
  char FNameFilter[81];
  char TempDir[MAXPATHLEN];
  char Dir[MAXPATHLEN];
  char Name[MAXPATHLEN];
  BOOL Ok;

  /* save current dir */
  getcwd(TempDir,sizeof(TempDir));

  /* File name filter */
  memset(FNameFilter, 0, sizeof(FNameFilter));
  strcpy(FNameFilter, "setup files(*.ini)");
  Ptr = strlen(FNameFilter) + 1;
  strcpy(&(FNameFilter[Ptr]), "*.ini"); 

  /* OPENFILENAME record */
  memset(&ofn, 0, sizeof(OPENFILENAME));

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner   = HWin;
  ofn.lpstrFile   = Name;
  ofn.nMaxFile    = sizeof(Name);
  ofn.lpstrFilter = FNameFilter;
  ofn.lpstrDefExt = "ini";
  ofn.lpTemplateName = MAKEINTRESOURCE(Id_FnDlg);
  ofn.hInstance = HInst;

  GetFileNamePosJ(ts->SetupFName,&i,&j);
  strcpy(Name,&(ts->SetupFName[j]));
  memcpy(Dir,ts->SetupFName,i);
  Dir[i] = 0;

  if ((strlen(Name)==0) || (stricmp(Name,"TERATERM.INI")==0))
  {
    strcpy(Name,"TERATERM.INI");
    ofn.nFilterIndex = 1;
  }
  else
    ofn.nFilterIndex = 2;

  if (strlen(Dir)==0)
    strcpy(Dir,ts->HomeDir);

  chdir(Dir);

  ofn.Flags = OFN_SHOWHELP | OFN_HIDEREADONLY | OFN_ENABLETEMPLATE;
  if (Save)
  {
    ofn.lpstrTitle = "Tera Term: Save setup";
    Ok = GetSaveFileName(&ofn);
  }
  else {
    ofn.Flags = ofn.Flags | OFN_FILEMUSTEXIST;
    ofn.lpstrTitle = "Tera Term: Restore setup";
    Ok = GetOpenFileName(&ofn);
  }

  if (Ok)
    strcpy(ts->SetupFName,Name);

  /* restore dir */
  chdir(TempDir);

  return Ok;
}

/* Hook function for file name dialog box */
BOOL CALLBACK TFnHook(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  LPOPENFILENAME ofn;
  WORD Lo, Hi;
  PLONG pl;

  switch (Message) {
    case WM_INITDIALOG:
      ofn = (LPOPENFILENAME)lParam;
      pl = (PLONG)(ofn->lCustData);
      SetWindowLong(Dialog, DWL_USER, (LONG)pl);
      Lo = LOWORD(*pl) & 1;
      Hi = HIWORD(*pl);
      ShowDlgItem(Dialog,Id_FnBinary,Id_FnBinary);
      SetRB(Dialog,Lo,Id_FnBinary,Id_FnBinary);
      if (Hi!=0xFFFF)
      {
        ShowDlgItem(Dialog,Id_FnAppend,Id_FnAppend);
        SetRB(Dialog,Hi & 1,Id_FnAppend,Id_FnAppend);
      }
      return TRUE;
    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  pl = (PLONG)GetWindowLong(Dialog,DWL_USER);
          if (pl!=NULL)
          {
            GetRB(Dialog,&Lo,Id_FnBinary,Id_FnBinary);
            Hi = HIWORD(pl);
            if (Hi!=0xFFFF)
              GetRB(Dialog,&Hi,Id_FnAppend,Id_FnAppend);
            *pl = MAKELONG(Lo,Hi);
          }
          break;
        case IDCANCEL:
	  break;
      }
  }
  return FALSE;
}

BOOL FAR PASCAL GetTransFname
  (HWND HWin, WORD FuncId, PLONG Option, PFileVar fv, PCHAR CurDir)
{
  char FNFilter[11];
  OPENFILENAME ofn;
  LONG opt;
  char TempDir[MAXPATHLEN];
  BOOL Ok;

  /* save current dir */
  getcwd(TempDir,sizeof(TempDir));

  fv->FullName[0] = 0;
  memset(FNFilter, 0, sizeof(FNFilter));  /* Set up for double null at end */
  memset(&ofn, 0, sizeof(OPENFILENAME));

  strcpy(fv->DlgCaption,"Tera Term: ");
  switch (FuncId) {
    case GTF_SEND:
      strcat(fv->DlgCaption,"Send file");
      break;
    case GTF_LOG:
      strcat(fv->DlgCaption,"Log");
      break;
    case GTF_BP:
      strcat(fv->DlgCaption,"B-Plus Send");
      break;
    default: return FALSE;
  }

  strcpy(FNFilter, "all");
  strcpy(&(FNFilter[strlen(FNFilter)+1]), "*.*");      

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner   = HWin;
  ofn.lpstrFilter = FNFilter;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = fv->FullName;
  ofn.nMaxFile = sizeof(fv->FullName);
  ofn.lpstrInitialDir = CurDir;
  ofn.Flags = OFN_SHOWHELP | OFN_HIDEREADONLY |
              OFN_ENABLETEMPLATE;
  if (FuncId!=GTF_BP)
  {
    ofn.Flags = ofn.Flags | OFN_ENABLEHOOK;
    ofn.lpfnHook = (LPOFNHOOKPROC)(&TFnHook);
  }
  opt = *Option;
  if (FuncId!=GTF_LOG)
  {
    ofn.Flags = ofn.Flags | OFN_FILEMUSTEXIST;
    opt = MAKELONG(LOWORD(*Option),0xFFFF);
  }
  ofn.lCustData = (DWORD)&opt;
  ofn.lpstrTitle = fv->DlgCaption;
  ofn.lpTemplateName = MAKEINTRESOURCE(Id_FnDlg);

  ofn.hInstance = HInst;
  Ok = GetOpenFileName(&ofn);
  if (Ok)
  {
    if (FuncId==GTF_LOG)
      *Option = opt;
    else
      *Option = MAKELONG(LOWORD(opt),HIWORD(*Option));

    fv->DirLen = ofn.nFileOffset;
    if (CurDir!=NULL)
    {
      memcpy(CurDir,fv->FullName,fv->DirLen-1);
      CurDir[fv->DirLen-1] = 0;
    }
  }
  /* restore dir */
  chdir(TempDir);
  return Ok;
}

BOOL CALLBACK TFn2Hook(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  LPOPENFILENAME ofn;
  PWORD pw;

  switch (Message) {
    case WM_INITDIALOG:
      ofn = (LPOPENFILENAME)lParam;
      pw = (PWORD)ofn->lCustData;
      SetWindowLong(Dialog, DWL_USER, (LONG)pw);
      ShowDlgItem(Dialog,Id_Fn2Binary,Id_Fn2Binary);
      SetRB(Dialog,*pw & 1,Id_Fn2Binary,Id_Fn2Binary);
      return TRUE;
    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  pw = (PWORD)GetWindowLong(Dialog,DWL_USER);
          if (pw!=NULL)
            GetRB(Dialog,pw,Id_Fn2Binary,Id_Fn2Binary);
          break;
        case IDCANCEL:
	  break;
      }
  }
  return FALSE;
}

BOOL FAR PASCAL GetMultiFname
  (HWND HWin, PFileVar fv, PCHAR CurDir, WORD FuncId, PWORD Option)
{
  int i;
  char FNFilter[11];
  OPENFILENAME ofn;
  char TempDir[MAXPATHLEN];
  BOOL Ok;

  /* save current dir */
  getcwd(TempDir,sizeof(TempDir));

  fv->NumFname = 0;

  strcpy(fv->DlgCaption,"Tera Term: ");
  switch (FuncId) {
    case GMF_KERMIT:
      strcat(fv->DlgCaption,"Kermit Send");
      break;
    case GMF_Z:
      strcat(fv->DlgCaption,"ZMODEM Send");
      break;
    case GMF_QV:
      strcat(fv->DlgCaption,"Quick-VAN Send");
      break;
    default: return FALSE;
  }

  fv->FnStrMemHandle = GlobalAlloc(GMEM_MOVEABLE, FnStrMemSize);
  if (fv->FnStrMemHandle == NULL)
  {
    MessageBeep(0);
    return FALSE;
  }
  else {
    fv->FnStrMem = GlobalLock(fv->FnStrMemHandle);
    if (fv->FnStrMem == NULL)
    {
      GlobalFree(fv->FnStrMemHandle);
      fv->FnStrMemHandle = 0;
      MessageBeep(0);
      return FALSE;
    }
  }

  fv->FnStrMem[0] = 0;

  memset(FNFilter, 0, sizeof(FNFilter));  /* Set up for double null at end */
  strcpy(FNFilter, "all");
  strcpy(&(FNFilter[strlen(FNFilter)+1]), "*.*");
  memset(&ofn, 0, sizeof(OPENFILENAME));

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner   = HWin;
  ofn.lpstrFilter = FNFilter;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = fv->FnStrMem;
  ofn.nMaxFile = FnStrMemSize;
  ofn.lpstrTitle= fv->DlgCaption;
  ofn.lpstrInitialDir = CurDir;
  ofn.Flags = OFN_SHOWHELP | OFN_ALLOWMULTISELECT |
             OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_ENABLETEMPLATE;
  ofn.lCustData = NULL;
  if (FuncId==GMF_Z)
  {
    ofn.Flags = ofn.Flags | OFN_ENABLEHOOK;
    ofn.lCustData = (DWORD)Option;
    ofn.lpfnHook = (LPOFNHOOKPROC)(&TFn2Hook);
  }
  ofn.lpTemplateName = MAKEINTRESOURCE(Id_FnDlg2);
  ofn.hInstance = HInst;

  Ok = GetOpenFileName(&ofn);

  if (Ok)
  {
    /* count number of file names */
    i = 0;
    while (fv->FnStrMem[i]!=0)
    {
      if (fv->FnStrMem[i]==' ')
      {
        fv->NumFname++;
        if (fv->NumFname == 1) fv->DirLen = i+1;
      }
      i++;
    }

    if (fv->NumFname<1)
    {
      fv->NumFname = 1;
      fv->DirLen = ofn.nFileOffset;
      strcpy(fv->FullName,fv->FnStrMem);
    }
    else {
      memcpy(fv->FullName,fv->FnStrMem,fv->DirLen-1);
      fv->FullName[fv->DirLen] = 0;
    }
    fv->FullName[fv->DirLen-1] = '\\';

    memcpy(CurDir,fv->FullName,fv->DirLen-1);
    CurDir[fv->DirLen-1] = 0;

    fv->FNCount = 0;
    if (fv->NumFname==1) fv->FnPtr = 0;
                    else fv->FnPtr = fv->DirLen;
  }

  GlobalUnlock(fv->FnStrMemHandle);
  if (! Ok)
  {
    GlobalFree(fv->FnStrMemHandle);
    fv->FnStrMemHandle = NULL;
  }

  /* restore dir */
  chdir(TempDir);

  return Ok;
}

BOOL CALLBACK GetFnDlg
  (HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  PFileVar fv;
  char TempFull[MAXPATHLEN];
  int i, j;

  switch (Message) {
    case WM_INITDIALOG:
      fv = (PFileVar)lParam;
      SetWindowLong(Dialog, DWL_USER, lParam);
      SendDlgItemMessage(Dialog, Id_GetFn, EM_LIMITTEXT, sizeof(TempFull)-1,0);
      return TRUE;
    case WM_COMMAND:
      fv = (PFileVar)GetWindowLong(Dialog,DWL_USER);
      switch (LOWORD(wParam)) {
        case IDOK:
          if (fv!=NULL)
          {
            GetDlgItemText(Dialog, Id_GetFn, TempFull, sizeof(TempFull));
            if (strlen(TempFull)==0) return TRUE;
            GetFileNamePosJ(TempFull,&i,&j);
            FitFileName(&(TempFull[j]),NULL);
            strcat(fv->FullName,&(TempFull[j]));
          }
          EndDialog(Dialog, 1);
          return TRUE;
        case IDCANCEL:
          EndDialog(Dialog, 0);
          return TRUE;
        case Id_GetFnHelp:
          if (fv!=NULL)
            PostMessage(fv->HMainWin,WM_USER+MsgDlgHelp2,0,0);
          break;
      }
  }
  return FALSE;
}

BOOL FAR PASCAL GetGetFname(HWND HWin, PFileVar fv)
{
  return
    (BOOL)DialogBoxParam(HInst,
          MAKEINTRESOURCE(Id_GetFnDlg),
          HWin, GetFnDlg, (LONG)fv);
}

void FAR PASCAL SetFileVar(PFileVar fv)
{
  int i;

  GetFileNamePosJ(fv->FullName,&(fv->DirLen),&i);
  if (fv->FullName[fv->DirLen]=='\\') fv->DirLen++;
  strcpy(fv->DlgCaption,"Tera Term: ");
  switch (fv->OpId) {
    case OpLog:
      strcat(fv->DlgCaption,TitLog);
      break;
    case OpSendFile:
      strcat(fv->DlgCaption,TitSendFile);
      break;
    case OpKmtRcv:
      strcat(fv->DlgCaption,TitKmtRcv);
      break;
    case OpKmtGet:
      strcat(fv->DlgCaption,TitKmtGet);
      break;
    case OpKmtSend:
      strcat(fv->DlgCaption,TitKmtSend);
      break;
    case OpKmtFin:
      strcat(fv->DlgCaption,TitKmtFin);
      break;
    case OpXRcv:
      strcat(fv->DlgCaption,TitXRcv);
      break;
    case OpXSend:
      strcat(fv->DlgCaption,TitXSend);
      break;
    case OpZRcv:
      strcat(fv->DlgCaption,TitZRcv);
      break;
    case OpZSend:
      strcat(fv->DlgCaption,TitZSend);
      break;
    case OpBPRcv:
      strcat(fv->DlgCaption,TitBPRcv);
      break;
    case OpBPSend:
      strcat(fv->DlgCaption,TitBPSend);
      break;
    case OpQVRcv:
      strcat(fv->DlgCaption,TitQVRcv);
      break;
    case OpQVSend:
      strcat(fv->DlgCaption,TitQVSend);
      break;
  }
}

/* Hook function for XMODEM file name dialog box */
BOOL CALLBACK XFnHook(HWND Dialog, UINT Message, WPARAM wParam, LPARAM lParam)
{
  LPOPENFILENAME ofn;
  WORD Hi, Lo;
  PLONG pl;

  switch (Message) {
    case WM_INITDIALOG:
      ofn = (LPOPENFILENAME)lParam;
      pl = (PLONG)ofn->lCustData;
      SetWindowLong(Dialog, DWL_USER, (LONG)pl);
      ShowDlgItem(Dialog,Id_FnBox,Id_FnX1K);
      SetRB(Dialog,HIWORD(*pl),Id_FnXCheck,Id_FnX1K);
      if (LOWORD(*pl)!=0xFFFF)
      {
        ShowDlgItem(Dialog,Id_FnBinary,Id_FnBinary);
        SetRB(Dialog,LOWORD(*pl),Id_FnBinary,Id_FnBinary);
      }
      return TRUE;
    case WM_COMMAND:
      switch (LOWORD(wParam)) {
        case IDOK:
	  pl = (PLONG)GetWindowLong(Dialog,DWL_USER);
          if (pl!=NULL)
          {
            GetRB(Dialog,&Hi,Id_FnXCheck,Id_FnX1K);
            if (LOWORD(*pl)==0xFFFF)
              Lo = 0xFFFF;
            else
              GetRB(Dialog,&Lo,Id_FnBinary,Id_FnBinary);
            *pl = MAKELONG(Lo,Hi);
          }
          break;
        case IDCANCEL:
          break;
      }
  }
  return FALSE;
}

BOOL FAR PASCAL GetXFname
  (HWND HWin, BOOL Receive, PLONG Option, PFileVar fv, PCHAR CurDir)
{
  char FNFilter[11];
  OPENFILENAME ofn;
  LONG opt;
  char TempDir[MAXPATHLEN];
  BOOL Ok;

  /* save current dir */
  getcwd(TempDir,sizeof(TempDir));

  fv->FullName[0] = 0;
  memset(FNFilter, 0, sizeof(FNFilter));  /* Set up for double null at end */
  memset(&ofn, 0, sizeof(OPENFILENAME));

  strcpy(fv->DlgCaption,"Tera Term: XMODEM ");
  if (Receive)
    strcat(fv->DlgCaption,"Receive");
  else
    strcat(fv->DlgCaption,"Send");

  strcpy(FNFilter, "all");
  strcpy(&(FNFilter[strlen(FNFilter)+1]), "*.*");      

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner   = HWin;
  ofn.lpstrFilter = FNFilter;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = fv->FullName;
  ofn.nMaxFile = sizeof(fv->FullName);
  ofn.lpstrInitialDir = CurDir;
  ofn.Flags = OFN_SHOWHELP | OFN_HIDEREADONLY |
             OFN_ENABLETEMPLATE | OFN_ENABLEHOOK;
  opt = *Option;
  if (! Receive)
  {
    ofn.Flags = ofn.Flags | OFN_FILEMUSTEXIST;
    opt = opt | 0xFFFF;
  }
  ofn.lCustData = (DWORD)&opt;

  ofn.lpstrTitle = fv->DlgCaption;
  ofn.lpfnHook = (LPOFNHOOKPROC)(&XFnHook);
  ofn.lpTemplateName = MAKEINTRESOURCE(Id_FnDlg);
  ofn.hInstance = HInst;

  Ok = GetOpenFileName(&ofn);

  if (Ok)
  {
    fv->DirLen = ofn.nFileOffset;
    fv->FnPtr = ofn.nFileOffset;
    memcpy(CurDir,fv->FullName,fv->DirLen-1);
    CurDir[fv->DirLen-1] = 0;

    if (Receive)
      *Option = opt;
    else
      *Option = MAKELONG(LOWORD(*Option),HIWORD(opt));
  }

  /* restore dir */
  chdir(TempDir);

  return Ok;
}

void FAR PASCAL ProtoInit(int Proto, PFileVar fv, PCHAR pv, PComVar cv, PTTSet ts)
{
  switch (Proto) {
    case PROTO_KMT:
      KmtInit(fv,(PKmtVar)pv,cv,ts);
      break;
    case PROTO_XM:
      XInit(fv,(PXVar)pv,cv,ts);
      break;
    case PROTO_ZM:
      ZInit(fv,(PZVar)pv,cv,ts);
      break;
    case PROTO_BP:
      BPInit(fv,(PBPVar)pv,cv,ts);
      break;
    case PROTO_QV:
      QVInit(fv,(PQVVar)pv,cv,ts);
      break;
  }
}

BOOL FAR PASCAL ProtoParse
  (int Proto, PFileVar fv, PCHAR pv, PComVar cv)
{
  BOOL Ok;

  switch (Proto) {
    case PROTO_KMT:
      Ok = KmtReadPacket(fv,(PKmtVar)pv,cv);
      break;
    case PROTO_XM:
      switch (((PXVar)pv)->XMode) {
	case IdXReceive:
	  Ok = XReadPacket(fv,(PXVar)pv,cv);
	  break;
	case IdXSend:
	  Ok = XSendPacket(fv,(PXVar)pv,cv);
	  break;
      }
      break;
    case PROTO_ZM:
      Ok = ZParse(fv,(PZVar)pv,cv);
      break;
    case PROTO_BP:
      Ok = BPParse(fv,(PBPVar)pv,cv);
      break;
    case PROTO_QV:
      switch (((PQVVar)pv)->QVMode) {
	case IdQVReceive:
	  Ok = QVReadPacket(fv,(PQVVar)pv,cv);
	case IdQVSend:
	  Ok = QVSendPacket(fv,(PQVVar)pv,cv);
      }
      break;
  }
  return Ok;
}

void FAR PASCAL ProtoTimeOutProc
  (int Proto, PFileVar fv, PCHAR pv, PComVar cv)
{
  switch (Proto) {
    case PROTO_KMT:
      KmtTimeOutProc(fv,(PKmtVar)pv,cv);
      break;
    case PROTO_XM:
      XTimeOutProc(fv,(PXVar)pv,cv);
      break;
    case PROTO_ZM:
      ZTimeOutProc(fv,(PZVar)pv,cv);
      break;
    case PROTO_BP:
      BPTimeOutProc(fv,(PBPVar)pv,cv);
      break;
    case PROTO_QV:
      QVTimeOutProc(fv,(PQVVar)pv,cv);
      break;
  }
}

BOOL FAR PASCAL ProtoCancel
  (int Proto, PFileVar fv, PCHAR pv, PComVar cv)
{
  switch (Proto) {
    case PROTO_KMT:
      KmtCancel(fv,(PKmtVar)pv,cv);
      break;
    case PROTO_XM:
      if (((PXVar)pv)->XMode==IdXReceive)
        XCancel(fv,(PXVar)pv,cv);
      break;
    case PROTO_ZM:
      ZCancel((PZVar)pv);
      break;
    case PROTO_BP:
      if (((PBPVar)pv)->BPState != BP_Failure)
      {
	BPCancel((PBPVar)pv);
        return FALSE;
      }
      break;
    case PROTO_QV:
      QVCancel(fv,(PQVVar)pv,cv);
      break;
  }
  return TRUE;
}

#pragma off (unreferenced);
int APIENTRY LibMain(HANDLE hdll,
                     DWORD  reason,
                     LPVOID reserved)
#pragma on (unreferenced);
{
  HInst = hdll;
  switch( reason ) { 
  case DLL_THREAD_ATTACH:
     /* do thread initialization */
    break;
  case DLL_THREAD_DETACH:
    /* do thread cleanup */
    break;
  case DLL_PROCESS_ATTACH:
     /* do process initialization */
    break;
  case DLL_PROCESS_DETACH:
    /* do process cleanup */
    break;
  }
   return( 1 );
 }

