/* Tera Term
 Copyright(C) 1994-1996 T. Teranishi
 All rights reserved. */

/* misc. routines  */
#include "teraterm.h"
#include "tttypes.h"

BOOL GetFileNamePosJ(PCHAR PathName, int *DirLen, int *FNPos)
{
  int i, j, k, L;
  BYTE b;

  *DirLen = 0;
  *FNPos = 0;
  if (PathName==NULL) return FALSE;
  L = strlen(PathName);
  i = 0;
  if ((L>=2) && (PathName[1]==':'))
    i = 2;
  if (PathName[i]==0x5c) i++;
  j = i;
  k = i;
  while (i<L)
  {
    b = PathName[i];
    i++;
    switch (b) {
      case '*':
      case ':': return FALSE;
      case 0x5c:
          j = i - 1;
          k = i;
          break;
      default:
        if ((b>=0x81)&&(b<=0x9f) ||
            (b>=0xe0)&&(b<=0xfc)) i++; /* SJIS first byte */
    }
  }
  *DirLen = j;
  *FNPos = k;
  return TRUE;
}

BOOL ExtractFileNameJ(PCHAR PathName, PCHAR FileName)
{
  int i, j;

  if (FileName==NULL) return FALSE;
  if (! GetFileNamePosJ(PathName,&i,&j)) return FALSE;
  strncpy(FileName,&PathName[j],MAXPATHLEN);
  if (strlen(FileName)>0)
    return TRUE;
  else
    return FALSE;
}

BOOL ExtractDirNameJ(PCHAR PathName, PCHAR DirName)
{
  int i, j;

  if (DirName==NULL) return FALSE;
  if (! GetFileNamePosJ(PathName,&i,&j)) return FALSE;
  memcpy(&DirName[0],&PathName[0],i);
  DirName[i] = 0;
  return TRUE;
}

/* fit a given file to the windows-filename format */
/* FileName must contain filename part only. */
void FitFileName(PCHAR FileName, PCHAR DefExt)
{
  int i, j, NumOfDots;
  char Temp[MAXPATHLEN];
  BYTE b;
  
  NumOfDots = 0;
  i = 0;
  j = 0;
  /* filename started with a dot is illeagal */
  if (FileName[0]=='.')
  {
    Temp[0] = '_';  /* insert an underscore char */
    j++;
  }
  do {
    b = FileName[i];
    i++;
    if (b=='.') NumOfDots++;
    if ((b!=0) && 
        (j < MAXPATHLEN-1))
    {
      Temp[j] = b;
      j++;
    }
  } while (b!=0);

  Temp[j] = 0;
  if ((NumOfDots==0) &&
      (DefExt!=NULL))
    strcat(Temp,DefExt); /* add the default extension */

  strcpy(FileName,Temp);
}

void Str2Hex(PCHAR Str, PCHAR Hex, int Len, int MaxHexLen, BOOL ConvSP)
{
  BYTE b, low;
  int i, j;

  if (ConvSP)
    low = 0x20;
  else
    low = 0x1F;

  j = 0;
  for (i=0; i<=Len-1; i++)
  {
    b = Str[i];
    if ((b!='$') && (b>low) && (b<0x7f))
    {
      if (j < MaxHexLen)
      {
        Hex[j] = b;
        j++;
      }
    }
    else {
      if (j < MaxHexLen-2)
      {
        Hex[j] = '$';
        j++;
        if (b<=0x9f)
          Hex[i] = (char)((b >> 4) + 0x30);
        else
          Hex[i] = (char)((b >> 4) + 0x37);
        j++;
        if ((b & 0x0f) <= 0x9)
          Hex[i] = (char)((b & 0x0f) + 0x30);
        else
          Hex[i] = (char)((b & 0x0f) + 0x37);
        j++;
      }
    }
  }
  Hex[j] = 0;
}

BYTE ConvHexChar(BYTE b)
{
  if ((b>='0') && (b<='9'))
    return (b - 0x30);
  else if ((b>='A') && (b<='F'))
    return (b - 0x37);
  else if ((b>='a') && (b<='f'))
    return (b - 0x57);
  else
    return 0;
}

int Hex2Str(PCHAR Hex, PCHAR Str, int MaxStrLen)
{
  BYTE b, c;
  int i, imax, j;

  j = 0;
  imax = strlen(Hex);
  i = 0;
  while ((i < imax) && (j<MaxStrLen))
  {
    b = Hex[i];
    if (b=='$')
    {
      i++;
      if (i < imax)
        c = Hex[i];
      else
        c = 0x30;
      b = ConvHexChar(c) << 4;
      i++;
      if (i < imax)
        c = Hex[i];
      else
        c = 0x30;
      b = b + ConvHexChar(c);
    };

    Str[j] = b;
    j++;
    i++;
  }
  return j;
}
